function analytger,x,y

;+
;============================================================================
;
;	function : analytger
;
;	purpose  : calculate linear regression line to data
;
;       calls    : length.pro
;
;	written :  cbeck@KIS 06/02
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 2) then begin
	print
	print, "usage:  erg = analytger(x,y)"
	print
	print, "	Calculate linear regression line to data."
        print, "        Returns vector(2), [slope,offset], corresponding to "
        print, "        y = erg(0) * x + erg(1)."
        print
	print, "	Arguments"
        print, "                Input :"
        print, "		x : independent variable"
        print, "                y : dependent variable, y = m*x + b"
        print
        print, "                Output :"
        print, "                erg = [slope,offset]"
        print
        return,0
endif
;==============================================================================
;-

a = fltarr(2)
N = length(x)

; mean values

xmean = mean(x)
ymean = mean(y)

sum1 =total( x*y )
sum2 =total( x^2. )

; divisor

fraction = sum2 - N*xmean^2

a(0) = ( sum1 - N*xmean*ymean )
a(1) = ( ymean*sum2 - xmean*sum1 )

a = a/float(fraction)

return,a
end
