function byte2signedint,array,xdim,ydim,swap = swap

;+
;============================================================================
;
;	function : byte2int
;
;	purpose  : transform data in byte format into two-dimensional
;                  signed integer array. Range of values -32768 +32767.
;
;	written :  cbeck@KIS 06/02
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 3) then begin
	print
	print, "usage:  i = byte2signedint(array,xdim,ydim,/swap)"
	print
	print, "	Transform byte array of 2 x N size into 2-d matrix"
        print, "        (numx x numy) with signed integer entries."
        print, "        xdim*ydim must be gt N."
        print
	print, "	Arguments"
        print, "                Input :"
        print, "		array : byte array of size 2 x N"
        print, "                xdim,ydim : dimensions of resulting matrix"
        print
        print, "                Output:"
        print, "                2-dim matrix of size (xdim x ydim), signed"
        print, "                integer entries."
        print
        print, "        Keywords:"
        print, "                /swap : exchange low and high byte"
        print
        return,0
endif
;==============================================================================
;-

;------------------------------------------------------------------------------
; initialize vector of data values
;------------------------------------------------------------------------------
fltvec = fltarr(float(xdim)*float(ydim))

; transform to integer
if n_elements(swap) eq 0 then begin
fltvec = array(0,*)*256+array(1,*)

;------------------------------------------------------------------------------
; if high byte gt 128 -> negative value
;------------------------------------------------------------------------------
kk = where(array(0,*) gt 128)
if kk(0) ne -1 then fltvec(kk) =  array(0,kk)*256+array(1,kk) - 65536
endif

;------------------------------------------------------------------------------
; if /swap is set, exchange high and low byte
;------------------------------------------------------------------------------
if n_elements(swap) ne 0 then begin
fltvec = array(1,*)*256+array(0,*)
kk = where(array(1,*) gt 128)
if kk(0) ne -1 then fltvec(kk) =  array(1,kk)*256+array(0,kk) - 65536
endif

;------------------------------------------------------------------------------
; initialize storage matrix for result
;------------------------------------------------------------------------------
image = intarr(xdim,ydim)

;------------------------------------------------------------------------------
; arrange data values in matrix form according to order
;------------------------------------------------------------------------------
for k = 0,ydim-1 do begin
image(*,k) = fltvec(k*float(xdim):(k+1.)*float(xdim)-1.)
endfor

;------------------------------------------------------------------------------
; return 2-d matrix (xdim x ydim)
;------------------------------------------------------------------------------
return,image

end
