function check_physicality,X, precise = precise

;+
;============================================================================
;
;	function  :  check_physicality
;
;	purpose   : control of physical reasonability of 4x4 matrix
;
;	author    : cbeck@kis 06/02
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 1) then begin
	print
	print, "usage:   erg = check_physicality(X,precise=10)
	print
	print, "	 Control physical reasonability of matrix by numerical"
        print, "         test. This is NO rigid procedure, unphysicality may"
        print, "         be not detected !"
	print
	print, "	Arguments"
        print, "                X : 4x4 Mueller matrix 
        print
        print, "         Keywords :"
        print, "              precise = floating number > 0 : in-/decrease"
        print, "                        number of used grid points, values"
        print, "                        > 1 increase grid points. Values"
        print, "                        gt 2 lead to a looooooong calculation."
        print
        return,0
endif
;==============================================================================
;-

if n_elements(precise) eq 0 then precise = 1

;------------------------------------------------------
; define grid
;------------------------------------------------------
theta = (findgen(400*precise)/400*precise*2.*!pi)-!pi
phi = findgen(1000*precise)/1000.*precise*2.*!pi
cosphi = cos(phi)
sinphi = sin(phi)

;------------------------------------------------------
; initialize minimal values
;------------------------------------------------------
pmin = 1
posmin = [0,0]
Iprimemin = 10
posimin = [0,0]

;------------------------------------------------------
; loop over grid
;------------------------------------------------------
for i = 0,length(theta)-1 do begin

    Q = cosphi*sin(theta(i))
    U = sinphi*sin(theta(i))
    V = cos(theta(i)) ;*(phi-phi+1.)
    
    Iprime = X(0,0)+X(1,0)*Q+X(2,0)*U+X(3,0)*V
    Qprime = X(0,1)+X(1,1)*Q+X(2,1)*U+X(3,1)*V
    Uprime = X(0,2)+X(1,2)*Q+X(2,2)*U+X(3,2)*V
    Vprime = X(0,3)+X(1,3)*Q+X(2,3)*U+X(3,3)*V
    P = Iprime^2.-(Qprime^2.+Uprime^2.+Vprime^2.)

;------------------------------------------------------
; find minimal values for P and Iprime
;------------------------------------------------------
    pp = min(P,posminp)
        if pp lt pmin then begin
        pmin = pp
        posmin = [phi(posminp),theta(i)]
    endif
    
    ii = min(Iprime,posmini)
    if ii lt Iprimemin then begin
        Iprimemin = ii
        posimin = [phi(posmini),theta(i)]
    endif
;------------------------------------------------------
; break loop if unphysicality is detected
;------------------------------------------------------
    if pmin lt 0 then break
    if Iprimemin lt 0 then break
endfor

;-------------------------------------------------------------------------
; return minimal values for P,phiminp,thetaminp,Iprime,phiminI,thetaminI
;-------------------------------------------------------------------------
return,[pmin,posmin(0),posmin(1),Iprimemin,posimin(0),posimin(1)]

end
