pro deskew1,icorr,qcorr,ucorr,vcorr,ilo,ihi,firsty,lasty,y1,y2,repetitions,posref,plt=plt,straight = straight,third_order = third_order,ll = ll

;+
;============================================================================
;
;	procedure : deskew1
;
;	purpose   : calculate position of line cores for image.
;	            Fit polynomial on course, shift all images to line
;	            core at slit height /4 .
; 
;       calls     : shift_img.pro, lpff.pro
;
;	written   : cbeck@KIS 06/02
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 11) then begin
	print
	print, "usage:  deskew1,icorr,qcorr,ucorr,vcorr,ilo,ihi,firsty,lasty$"
        print, "                  ,y1,y2,repetitions,/plt,straight = straight"
	print
	print, "	Deskew spectral images. Reference value is line core"
        print, "        of spectral line at 1/4 slit height."
	print
	print, "	Arguments"
        print, "                Input :"
	print, "		icorr,qcorr,ucorr,vcorr : polarization images"
        print, "                ilo,ihi   : range of spectral line to be"
        print, "                            used in aligning"
        print, "                firsty,lasty : useable range along slit"
        print, "                y1,y2     : full data range"
        print, "                repetitions  : number of iterative repetitions"
        print, "                               to improve accuracy"
        print, "        Keywords"
        print, "                plt : plot line cores before and"
        print, "                      after shift"
        print, "                straight : use only 1st degree polynomial"
	print
        return
endif
;==============================================================================
;-

if n_elements(ll) eq 0 then ll = 5

;if n_elements(plt) eq 0 then plt = 0
if n_elements(third_order) eq 0 then grade = 2 else grade = 3

ssize = size(icorr)

;--------------------------------------------------
; loop over number of repetitions
;--------------------------------------------------
for rep = 0,repetitions do begin 

;--------------------------------------------------
; get position of line core in Stokes I
; find minimum inside range
;--------------------------------------------------
    pos_range = fltarr(lasty-firsty+1)
    for k = firsty,lasty do begin
        gg = min(icorr(ilo:ihi,k-firsty),posmin)
        pos_range(k-firsty) = posmin
    endfor
    posa = fltarr(lasty-firsty+1)

;---------------------------------------------------------------
; get line core position with lpff.pro around minimum intensity
; calculate curvature only between firsty:lasty
;---------------------------------------------------------------
    for k = firsty,lasty do begin
       if ilo+pos_range(k-firsty)+ll ge ssize(1) then pos_up = ssize(1)-1 else pos_up = ilo+pos_range(k-firsty)+ll

        lpff,icorr(ilo+pos_range(k-firsty)-ll:pos_up,k-firsty),pos

        posa(k-firsty) = pos+pos_range(k-firsty)-ll
    endfor

;--------------------------------------------------------------
; fit polynomial of 2nd order to position of core along slit
; extend curve to full data range y1:y2
;--------------------------------------------------------------
    range_fit = findgen(lasty-firsty+1)+firsty 
    range_corr = findgen(y2-y1+1)

if n_elements(straight) eq 0 then begin
    erga = poly_fit(range_fit(1:lasty-firsty-1),posa(1:lasty-firsty-1)+ilo,grade,/double)
    yy_a = erga(0)+erga(1)*range_fit+erga(2)*(range_fit)^2.
    if grade eq 3 then  yy_a = yy_a +erga(3)*(range_fit)^3.
    if grade eq 4 then  yy_a = yy_a +erga(4)*(range_fit)^4.
    yy_corr = erga(0)+erga(1)*range_corr+erga(2)*(range_corr)^2.
    if grade eq 3 then  yy_corr = yy_corr +erga(3)*(range_corr)^3.
    if grade eq 4 then  yy_corr = yy_corr +erga(4)*(range_corr)^4.
endif else begin
    ergb = poly_fit(range_fit(1:lasty-firsty-1),posa(1:lasty-firsty-1)+ilo,1,/double)
    yy_a = ergb(0)+ergb(1)*range_fit
    yy_corr = ergb(0)+ergb(1)*range_corr
endelse



posref = yy_a((lasty-firsty)/4)

;--------------------------------------------------------------------
; get difference of line cores to fixed value at 1/4 slit height
;--------------------------------------------------------------------
    ydiff = yy_corr-yy_a((lasty-firsty)/4)

rr= mean(ydiff)
if rr gt 7.5 then begin
    print,' deskew1.pro: '
    print,' Warning ! Very high mean shift value of '+num2string(rr)+' pixels.'
    print,' Suggestion: check range of spectral lines/hairlines.'
    if n_elements(plt) ne 0 then begin
      ;  print,' Do you want plotting to be switched on ?'
      ;  ans = read_keyboard(' (y)es or (n)o : ')
      ;  if ans eq 'y' then plt = 1
    endif
endif
;--------------------------------------------------
; do plotting
;--------------------------------------------------
    if n_elements(plt) ne 0 then begin
        erase
        plot,range_fit,posa+ilo,/ynozero,title = 'Skewness, shift : '+num2string(rep),xtitle=' Slit height', ytitle=' Pos. of line core / pixel ',xcharsize=1.3,ycharsize=1.5,charsize=1.5
        oplot,range_fit,yy_a
        print,mean(ydiff),' Mean difference of line cores in pixel.'
;        paused
    endif

;-------------------------------------------------------------------------
; shift all images to the default position at 1/4 slit height
;-------------------------------------------------------------------------
shift_img,icorr,-ydiff,y1,y2-1
shift_img,qcorr,-ydiff,y1,y2-1
shift_img,ucorr,-ydiff,y1,y2-1
shift_img,vcorr,-ydiff,y1,y2-1

endfor

;--------------------------------------------------
; display final result (same steps as above)
;--------------------------------------------------
if n_elements(plt) ne 0 then begin
    pos_range = fltarr(lasty-firsty+1)
    for k = firsty,lasty do begin
        gg = min(icorr(ilo:ihi,k-firsty),posmin)
        pos_range(k-firsty) = posmin
    endfor

    posa = fltarr(lasty-firsty+1)
    for k = firsty,lasty do begin
       if ilo+pos_range(k-firsty)+ll ge ssize(1) then pos_up = ssize(1)-1 else pos_up = ilo+pos_range(k-firsty)+ll
        lpff,icorr(ilo+pos_range(k-firsty)-ll:pos_up,k-firsty),pos
        posa(k-firsty) = pos+pos_range(k-firsty)-5
    endfor

    range_fit = findgen(lasty-firsty+1)+firsty 

    if n_elements(straight) eq 0 then begin
        erga = poly_fit(range_fit(1:lasty-firsty-1),posa(1:lasty-firsty-1)+ilo,grade,/double)
        yy_a = erga(0)+erga(1)*range_fit+erga(2)*(range_fit)^2.
        if grade eq 3 then  yy_a = yy_a +erga(3)*(range_fit)^3.
        if grade eq 4 then  yy_a = yy_a +erga(4)*(range_fit)^4.
        endif else begin
            ergb = poly_fit(range_fit(1:lasty-firsty-1),posa(1:lasty-firsty-1)+ilo,1,/double)
            yy_a = ergb(0)+ergb(1)*range_fit
            yy_corr = ergb(0)+ergb(1)*range_corr
        endelse
        
ydiff = yy_a-yy_a((lasty-firsty)/4)

;--------------------------------------------------
; do plotting
;--------------------------------------------------
    plot,range_fit,posa+ilo,/ynozero,title = 'Line cores, final shift',xtitle=' Slit height', ytitle=' Pos. of line core / pixel',xcharsize=1.3,ycharsize=1.5,charsize=1.5
    oplot,range_fit,yy_a
    print,mean(ydiff),' Mean difference of line cores in pixel.'
    ;paused
endif

;--------------------------------------------------
; Done.
;--------------------------------------------------

end
