pro draw_poly,x1,y1,index,normal = normal,device = device,data = data,line_fill = line_fill,orientation = orientation,spacing = spacing,invisible = invisible,t3d = t3d,z1 = z1,thick = thick

;+
;============================================================================
;
;	procedure : draw_poly
;
;	purpose   : draw (filled) polygon given by x_i,y_i (z_i) vertices
;
;	written :  cbeck@KIS2004
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 2) then begin
	print
	print, "usage:  draw_poly,x1,y1,index,normal = normal,device = device"
        print, "                 ,data = data,line_fill = line_fill"
        print, "                 ,orientation = orientation,spacing = spacing"
        print, "                 ,invisible = invisible,t3d = t3d,z1 = z1"
	print
	print, "	Draw polygon to the vertices [x1_i,y1_i]. Polygon may"
        print, "        completely filled, filled with line of a certain"
        print, "        spacing, or only the contours are plotted."
        print, "        In 3-D mode, z1 is the third coordinate."
        print
	print, "	Arguments"
        print, "                Input :"
        print, "		x1,y1 : vectors of the same length, containing"
        print, "                        the position of the corners."
        print, "                index : color of polygon or lines."
        print
        print, "        Keywords:"
        print, "                normal,device,data : as usual"
        print, "                line_fill   : fill with lines instead of solid"
        print, "                orientation : orientation of the lines in deg"
        print, "                spacing     : distance between two lines"
        print, "                invisible   : draw only outlines of polygon"
        print, "                /t3d        : activate 3-D perspective"
        print, "                z1          : z-coordinate, same size as x1,y1"
        print
        return
endif
;==============================================================================
;-

if n_elements(invisible) eq 0 then invisible = 0

if n_elements(t3d) eq 0 then begin
    if invisible eq 0 then begin
        oROI = OBJ_NEW('IDLanROI', TYPE=2) 
        for kk = 0,n_elements(x1)-1 do begin
            oROI->AppendData, x1(kk), y1(kk)
        endfor
        DRAW_ROI, oROI, normal = normal,device = device ,color =index,data = data,line_fill = line_fill,orientation = orientation,spacing = spacing
    endif else begin
        plots,x1,y1,normal = normal,device = device ,color =index,data = data,thick = thick
    endelse
endif else begin
    oROI = OBJ_NEW('IDLanROI', TYPE=2) 
    for kk = 0,n_elements(x1)-1 do begin
        oROI->AppendData, x1(kk), y1(kk),z1(kk)
    endfor
    if invisible eq 0 then DRAW_ROI, oROI, normal = normal,device = device ,color =index,data = data,line_fill = line_fill,orientation = orientation,spacing = spacing,t3d = t3d else plots,x1,y1,z1,normal = normal,device = device ,color =index,data = data,thick = thick
    
endelse
 
end
