function histogra2d,x,y1,bins

;+
;============================================================================
;
;	function : histogra2d
;
;	purpose  : calculate histogram of vector x into n equidistant bins.
;                  Additionally averages values of dependent variable
;                  y1 in the bins.
;
;	written :  cbeck@KIS
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 3) then begin
	print
	print, "usage:  erg = histogra(x,y1,bins)"
	print
	print, "	Calculate histogram of values in x into #bins, which"
        print, "        are equidistant between max and min of x."
        print, "        Additionally calculate the average value of the"
        print, "        dependent variable y1 = f(x) in the bins."
        print
	print, "	Arguments"
        print, "                Input :"
        print, "		x : vector of values"
        print, "                bins : number of bins to use"
        print
        print, "                Output :"
        print, "                erg(0:bins-1) = histogram values in bins"
        print, "                erg(bins:bins*2-1) = central position of bins"
        print, "                erg(bins*2:bins*3-1) = average values of y1"
        print, "                erg(bins*3:bins*4-1) = rms of y1"
        print
        return,0
endif
;==============================================================================
;-


; Berechnung des Histogramms eines Vektors in der Anzahl von bins zwischen
; Maximal - und Minimalwert 
; Output : Vektor mit Laenge 2*bins ( Anzahl im bin, Binwert )

; Arbeitsvariable

y = float(x)

bins =float(bins)

ymax = max(y)
ymin = min(y)
ywidth =float(abs(ymax-ymin))/bins

rr = fltarr(bins)
rr2 = rr
rms = rr2

; Shift des Vektors auf groesser gleich 0
if (ymin le 0 ) then y = y + abs(ymin)
if (ymin gt 0 ) then y = y - ymin

; Bestimmung der Anzahl der Werte von y-k*xwidth kleiner 0
for t=0.,bins-1 do begin

ff = where( y ge t*ywidth  and  y lt (t+1)*ywidth)
if ff(0) ne -1 then begin
rr2(t) = mean(y1(ff))
if n_elements(ff) gt 3 then rms(t) = stddev(y1(ff))/sqrt(float(n_elements(ff)))
endif


n2 = length(where( y - (t+1.)*ywidth le 0 ))
rr(t) = n2

endfor

; Berechnung der Werte in den einzelnen Bins durch Subtraktion der
; Werte in den daruntenliegenden Bins
for t = 1.,bins-1 do begin
rr(t) = rr(t)-total(rr(0:t-1))
endfor

; Erzeugung der Werte der einzelnen Bins als x-Vektor
yscale = ymin + float(findgen(bins))*ywidth

; Uebergabe der Daten : in der 1. Haelfte des Vektors stehen die Werte
; in den Bins, in der 2.Haelfte der Wert des bins
rr1 = fltarr(4*bins)
rr1(0:bins-1) = rr(*)
rr1(bins:bins*2-1) = yscale
rr1(bins*2:bins*3-1) = rr2
rr1(bins*3:bins*4-1) = rms


return,rr1
end
