pro makecontplot,image,levels,xy,xy_info,index_do,offsset = offsset, scale = scale,color = color, thick = thick,gband = gband,filled = filled,mask = mask,up_lim = up_lim,noplot = noplot,ps = ps,noshort = noshort,no_edge=no_edge,close = close,linestyle = linestyle,too_long = too_long

;+
;============================================================================
;
;	procedure : makecontplot
;
;	purpose  : Overplot contours of image array. Additional
;	           options: remove short contours, fill contours,
;                  create separate mask of contours, overplot contours
;                  somewhere in graph.
;
;	written :  cbeck@KIS
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 2) then begin
	print
	print, "usage:  makecontplot,image,levels,xy,xy_info,index_do,"
        print, "                    offsset = offsset, scale = scale,"
        print, "                  color = color, thick = thick,gband = gband,"
        print,"                  filled = filled,mask = mask,up_lim = up_lim,"
        print,"                  noplot = noplot,ps = ps,noshort = noshort,"
        print, "                 no_edge=no_edge,close = close"
	print
	print, "	Plot contour lines of image either filled or unfilled."
        print, "        Omit short contour lines or create mask of contours."
        print
	print, "	Arguments"
        print, "                Input :"
        print, "		image : 2-D matrix"
        print, "                levels : contour levels to be drawn"
        print, "                xy,xy_info: same as for contour"
        print, "                index_do : cf. line 217, can be used to"
        print, "                           manually remove some contours"
        print
        print, "        Keywords:"
        print, "                offsset : offset [x0,y0] of lower left corner"
        print, "                          in normal coordinates."
        print, "                scale : [nx,ny] number of images in x/y"
        print, "                        dimension."
        print, "                E.g., to overplot contours of image to"
        print, "                a set of other images, set scale = [2,1]"
        print, "                (2 images in one row), and offsset to"
        print, "                [.5,.0] to overplot on the right image."
        print, "                color,thick : as usual"
        print, "                gband : lower limit of length of contours"
        print, "                        to be excluded. E.g.: all contours"
        print, "                        with < gband vertex points will be"
        print, "                        omitted."
        print, "                /filled : fill contours using draw_poly."
        print, "                mask : named variable of same dimensions"
        print, "                       as image, will contain a mask of the"
        print, "                       contours."
        print, "                up_lim : too short contours will be taken"
        print, "                         if their mean intensity is larger"
        print, "                         than up_lim * I_surroundings"
        print, "                /no_plot : suppress plotting"
        print, "                /ps : prevents errors using device in ps-mode"
        print, "                /noshort : throw out too short contours"
        print, "                no_edge,close : see contour."
        print
        print, " To create a mask of the contours use"
        print, "    ...,gband = 15,/filled,mask = mask"
        print
        return
endif
;==============================================================================
;-

;,xy,xy_info,levels,noiso = noiso


if n_elements(no_edge) ne 0 then close = 0 else close = 1
if n_elements(up_lim) eq 0 then up_lim = 1.5
if n_elements(offsset) eq 0 then offsset = [0.,0.]
if n_elements(scale) eq 0 then scale = [1.,1.] else scale = float(scale)
if n_elements(gband) ne 0 then begin
do_gband = 1 
nn_limit = gband
endif else do_gband = 0
if n_elements(too_long) eq 0 then too_long = 10000.

if n_elements(color) gt 1 then begin
colorindex = 0
endif

tt = float(size(image))

contour,image,path_xy = xy1, path_info = xy_info1,levels = levels,nlevels = n_elements(levels)-1,close = close

if n_elements(xy1) eq 0 then begin
print,'!makecontplot: no contours found.'
print,'Returning.'
return
endif

xx = xy1(0,*)*tt(1)
yy = xy1(1,*)*tt(2)

;print,min(xx),max(xx) ;,min(yy),max(yy)
;print,min(xx)/tt(2),max(xx)/tt(2)


xx = xx -min(xx) ;-30.
yy = yy -min(yy) ;-40.

xx = xx/float(max(xx))
yy = yy/float(max(yy))

xy1(0,*) = xx/scale(0)+offsset(0)-.001
xy1(1,*) = yy/scale(1)+offsset(1)-.001

if n_elements(ps) eq 0 then begin ;
index = 10
device,decomposed = 0
loadct,13,/silent
endif


if n_elements(ps) ne 0 then begin
;loadct,13,/silent
index = 10
endif

if do_gband eq 0 then begin
    if n_elements(noplot) eq 0 then begin

        for k = 0,n_elements(xy_info1)-1 do begin
            s = [indgen((xy_info1(k).N)),0]
            plots,xy1(*,xy_info1(k).offset+S),/normal,thick = thick,color = color,linestyle = linestyle
        endfor
    endif

endif else begin
    if n_elements(noplot) eq 0 then begin
        for k = 0,n_elements(xy_info1)-1 do begin
            nn = xy_info1(k).N
            if nn gt nn_limit and nn lt too_long then begin
                s = [indgen((xy_info1(k).N)),0]
                if n_elements(filled) ne 0 then begin
                    x1 = xy1(0,xy_info1(k).offset+S)
                    y1 = xy1(1,xy_info1(k).offset+S)
                    if n_elements(color) eq 1 then draw_poly,x1,y1,color,/normal
                    if n_elements(color) gt 1 then begin
                        draw_poly,x1,y1,color(colorindex),/normal
                        colorindex = colorindex+1
                        if colorindex gt n_elements(color)-1 then colorindex =n_elements(color)-1 
                    endif

                endif else begin
                    if n_elements(color) gt 1 then begin
                        plots,xy1(*,xy_info1(k).offset+S),/normal,thick = thick,color = color(colorindex),linestyle = linestyle
                        colorindex = colorindex+1
                        if colorindex gt n_elements(color)-1 then colorindex =n_elements(color)-1 
                    endif else plots,xy1(*,xy_info1(k).offset+S),/normal,thick = thick,color = color,linestyle = linestyle
                endelse
            endif
        endfor
    endif

endelse

if do_gband eq 1 and n_elements(ps) eq 0 then begin
    print,' Masking all contours...'
    window,17,xsize=scale(0)*tt(1),ysize = scale(1)*tt(2)
; do full mask
    loadct,0,/silent
    for k = 0,n_elements(xy_info1)-1 do begin
        nn = xy_info1(k).N
        s = [indgen((xy_info1(k).N)),0]
        x1 = xy1(0,xy_info1(k).offset+S)
        y1 = xy1(1,xy_info1(k).offset+S)
        draw_poly,x1,y1,index,/normal
        plots,xy1(*,xy_info1(k).offset+S),/normal,thick = 2.,color = index
        index = index+1
    endfor        
    
    xoff = offsset(0)*scale(0)*tt(1) 
    yoff = offsset(1)*scale(1)*tt(2) 
    a = tvrd(xoff,yoff,tt(1),tt(2))
    fullmask = a

if n_elements(xy_info1) gt 254 then begin
print,' ! Warning ! Too many contours: '+num2string(fix(n_elements(xy_info1)))
print,' Maximum number allowed: 254'
print,' No guarantee for results !'
xy_info1 = xy_info1(0:253)
endif

   ; erase
   ; tvscl,a
   ; paused
    wdelete,17
    index_do = fltarr(n_elements(xy_info1)+1)
    for k = 10.,n_elements(xy_info1)+10. do begin
        ttt = where(round(fullmask) eq k)
        if ttt(0) ne -1 then begin
            
            nnn = length(ttt)
            xran = fltarr(nnn) 
            yran = fltarr(nnn) 
            for kk = 0.,nnn-1 do begin
                xran(kk)  = ttt(kk) mod fix(tt(1))
                yran(kk)  = ttt(kk) / fix(tt(1))
            endfor
            
            xmax = max(xran) & ymax = max(yran)
            xmin = min(xran) & ymin = min(yran)

            imean = max(image(ttt))
            xminold  =xmin
            yminold  =ymin
           
            isurr = image(xmin:xmax,ymin:ymax)
            for kk = 0.,nnn-1 do begin
                isurr(xran(kk)-xmin,yran(kk)-ymin) = 1500000.
            endfor
                     
            ttt = where(isurr ne 1500000.)
            if ttt(0) ne -1 then isurr = max(isurr(ttt)) else isurr =  1500000.
           ; 
           
;            if imean gt levels(0)*up_lim then begin
;                index_do(k-10) = 1
;            endif

            if imean gt isurr*up_lim then begin
                index_do(k-10) = 1
            endif
        endif
    endfor

; draw only contours longer than limit or with higher intensity
if n_elements(ps) eq 0 and n_elements(filled) eq 0 then begin ;
device,decomposed = 1
loadct,0,/silent
endif


if do_gband eq -1975. then begin
window,27,xsize= tt(1),ysize= tt(2)
ff = where(image gt  levels(0))
imgtemp = image-image
imgtemp(ff) = 1
window,28,xsize= tt(1),ysize= tt(2)
tvscl,imgtemp
index_do_old = index_do
print,' Left mouse button: keep structure.'
print,' Right mouse button: exclude structure.'
wset,27
for k = 0,n_elements(xy_info1)-1 do begin
erase
    nn = xy_info1(k).N
    if nn gt nn_limit and nn lt too_long then index_do(k) = 1
    print,nn,index_do(k)
    if index_do(k) ne 0 then begin
    s = [indgen((xy_info1(k).N)),0]
    x1 = xy1(0,xy_info1(k).offset+S)*2
    y1 = xy1(1,xy_info1(k).offset+S)
    draw_poly,x1,y1, 16777215,/normal
    plots, x1,y1,/normal,thick = 1,color =16777215
    cursor,x,y,device,/wait
    if !mouse.button eq 1 then index_do(k) = 1
    if !mouse.button eq 4 then index_do(k) = 0
    wait,.2
endif
endfor
endif

if n_elements(noplot) eq 0 then begin
    print,' Drawing selected contours...'
    if n_elements(noshort) ne 0 then index_do = index_do-index_do
    index = 10
    for k = 0,n_elements(xy_info1)-1 do begin
        nn = xy_info1(k).N

        if nn le nn_limit and index_do(k) eq 1 then nn = nn_limit +1

        ;if index_do(k) eq 0 then nn = nn_limit -1

        if nn gt nn_limit and nn lt too_long then begin  ;(nn gt nn_limit) or
        s = [indgen((xy_info1(k).N)),0]
        ; draw filled polygons instead of contour lines
        if n_elements(filled) ne 0 then begin
            x1 = xy1(0,xy_info1(k).offset+S)
            y1 = xy1(1,xy_info1(k).offset+S)
            draw_poly,x1,y1,index,/normal
            plots,xy1(*,xy_info1(k).offset+S),/normal,thick = thick,color = index
            index = index+5
        endif else plots,xy1(*,xy_info1(k).offset+S),/normal,thick = thick,color = color ; plot the contour lines only
    endif
endfor

endif

endif

if n_elements(filled) ne 0 and do_gband eq 1 and n_elements(ps) eq 0 then begin
scale1 = 1.

if n_elements(noshort) ne 0 then index_do = index_do-index_do

    print,' Making mask of selected contours...'
    index = 10
    window,17,xsize=scale(0)*tt(1),ysize=scale(1)*tt(2)
    device,decomposed = 1
    loadct,0,/silent
    for k = 0,n_elements(xy_info1)-1 do begin
        nn = xy_info1(k).N

        if nn le nn_limit and index_do(k) eq 1 then nn = nn_limit +1
        ;if index_do(k) eq 0 then nn = nn_limit -1
        
        if  nn gt nn_limit and nn lt too_long then begin ; or (index_do(k))) eq 1
            s = [indgen((xy_info1(k).N)),0]
       ; draw filled polygons instead of contour lines
        if n_elements(filled) ne 0 then begin
            x1 = xy1(0,xy_info1(k).offset+S)
            y1 = xy1(1,xy_info1(k).offset+S)

            xmin = min(x1) & xmax = max(x1)
            ymin = min(y1) & ymax = max(y1)
       
            x1 = x1  - xmin & y1 = y1 - ymin
            x1 = x1 * scale1 & y1 = y1 * scale1
           
            x1 = x1 + xmin + (1-scale1)/2.*(xmax-xmin)
            y1 = y1 + ymin + (1-scale1)/2.*(ymax-ymin)
           
            draw_poly,x1,y1,index,/normal
 
            index = index+1
        endif
       
    endif
endfor

xoff = offsset(0)*scale(0)*tt(1) 
yoff = offsset(1)*scale(1)*tt(2)

a = tvrd(xoff,yoff,tt(1),tt(2))
mask = a

wdelete,17
endif

xy_info = xy_info1
xy = xy1

if n_elements(ps) eq 0 then begin
device,decomposed = 1
loadct,0,/silent
endif


end
