pro read_reduced_spinor,infile,step,i,q,u,v,dx_final,dy_final,swap = swap

;+
; 
; procedure read_reduced_spinor
;
; Calling sequence: read_reduced_spinor,infile,step,i,q,u,v,dx_final,dy_final,swap = swap
;
; Purpose:
; 
; read in reduced unformatted SPINOR data. 
;
; Input:
;
; infile: file name of data file
; step:   scan step to be read
; dx_final,dy_final: dimensions of data. Dimensions of data are
; taken from the addtional save file ... .dat.sav, if not provided
;
; Output:
;
; i,q,u,v: Stokes IQUV images of one scan step
; dx_final,dy_final: as read in from the save file 
;
; Keywords:
;
; /swap: do a swap_endian of the data 
;-

if n_elements(dx_final) eq 0 or n_elements(dy_final) eq 0 then restore,infile+'.sav'

a = fltarr(dx_final+1,dy_final+1,4)
img = associate(infile,a,step)
if n_elements(swap) ne 0 then img = swap_endian(img)

i = reform(a(*,*,0))
q = reform(a(*,*,1))
u = reform(a(*,*,2))
v = reform(a(*,*,3))

end
