pro shift_img,img,shift_vec,firsty,lasty

;+
;============================================================================
;
;	procedure : shift_img
;
;	purpose  :  shift image row by row for amount in shift_vec
;
;       calls    :  shift_frac.pro
; 
;	written  :  cbeck@kis 06/02
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if (n_params() lt 4) then begin
	print
	print, "usage:  shift_img,img,shift_vec,firsty,lasty
	print
	print, "	Shift image row by row for amount in shift_vec."
	print
	print, "	Arguments"
        print, "                Input :"
	print, "		img       : image to be shifted"
        print, "                shift_vec : amount to be shifted in pixel,"
        print, "                            length must correspond with"
        print, "                            firsty,lasty"
        print, "                firsty,lasty : useable range along slit"
	print
        return
endif
;==============================================================================
;-

;---------------------------------------------------------------------
; shift row k by the amount given by shift_vec(k) using shift_bicub
; remark : shift_frac can only shift arrays, therefore the extension 
; to two rows, where only one is used afterwards
;---------------------------------------------------------------------
for k = firsty,lasty do begin
    gg = shift_frac(img(*,k:k+1),shift_vec(k-firsty),0,'C')
    img(*,k) = gg(*,0)
endfor

;-------------------------------------
; Done.
;-------------------------------------

end
