pro spinor_corr_fringes_new,infile,input_dir = input_dir,output_dir = output_dir,pposi = pposi,pposq = pposq,pposu = pposu,pposv = pposv,pos_solar = pos_solar,cont_wl =cont_wl,pos_lines = pos_lines,check_lines = check_lines,plt = plt,noiquv = noiquv,step = step,byline = byline,poscont = poscont,resize = resize,rresize = rresize,help = help

;+
;============================================================================
;
;	procedure : spinor_corr_fringes  
;
;	purpose   : remove fringes in QUV SPINOR observational data
;
;	author    : cbeck@nso Jul 2013
;
;==============================================================================
;
;	Check number of arguments.
;
;==============================================================================
if n_elements(help) ne 0 then begin
	print
	print, "usage:   spinor_corr_fringes,infile,input_dir = input_dir,output_dir = output_dir,ppos = ppos,pos_solar = pos_solar,cont_wl =cont_wl,pos_lines = pos_lines,check_lines = check_lines,plt = plt,noquv = noquv,step = step,help = help"
	print
	print, "	 Remove fringes in QUV SPINOR data."
        print, "         "
	print
	print, "	Arguments"
        print, "                infile: file name of input file to be corrected"
        print
        print, "         Keywords :"
        print, "              input_dir : source directory with reduced data."
        print, "              output_dir: destination directory for fringe-filtered reduced data."
        print, "              ppos      : fltarr(4) or fltarr(8) or fltarr(12); positions of frequencies"
        print, "                          to be removed, in the form [x0, x1 y0, y1], or"
        print, "                          [x0, x1 y0, y1,x2,x3,y2,x3], etc."
        print, "              pos_solar : location of solar line in pixels for"
        print, "                          creation of |QUV| maps. If not provided,"
        print, "                          a range around the solar lines will be used."
        print, "              cont_wl   : fltarr(2), [l0,l1] : wavelength pixel"
        print, "                          range to use constructing Ic map"
        print, "               pos_lines:fltarr(2),location of spectral lines in pixel"
        print, "              /check_lines: switch to enforce the manual selection"
        print, "                         of the spectral lines."
        print, "              /plt        : switch on plotting (recommended)"
        print, "              noquv       : fltarr(3); switch to deactivate filtering in in"
        print, "                            Stokes parameter. Default: [1,1,1], set to"
        print, "                            e.g. [0,1,1] to exclude Stokes Q from filtering"
        print, "              step        : scan step to be used for testing if first scan"
        print, "                            step is unsuited"
        print, "               /help   : switch to active the help text above."
        print
return
endif
;==============================================================================
;-


; see spinor_obs.pro for more details on calculation steps below, most
; of it is copy & paste from there.

if n_elements(noiquv) eq 0 then noiquv = [1,1,1,1]
if n_elements(output_dir) eq 0 then output_dir = '/nfs/export/scr2/cbeck/dpc/fringe_filtered_8542_spinor/072913/'
if n_elements(input_dir) eq 0 then  input_dir ='/nfs/export/scr2/cbeck/dpc/072913/reduced_8542_spinor/'
if n_elements(infile) eq 0 then infile = '130729.114800.0.ccc01.c-hrt.map.0005.fits.reduced.dat'

restore,input_dir+infile+'.sav'

; read scan step to test filter
if n_elements(step) eq 0 then step = 0
read_reduced_spinor,input_dir+infile,step,i,q,u,v,dx_final,dy_final,swap = swap

ff = where(finite(i) ne 1)
if ff(0) ne -1 then i(ff) = 1.

ff = where(finite(q) ne 1)
if ff(0) ne -1 then q(ff) = 0.

ff = where(finite(u) ne 1)
if ff(0) ne -1 then u(ff) = 0.

ff = where(finite(v) ne 1)
if ff(0) ne -1 then v(ff) = 0.

if n_elements(resize) ne 0 then begin
   iorig = i
   tt = size(iorig)
   xdimorig = tt(1)
   ydimorig = tt(2)

   i = i(*,0:tt(2)-30)
   q = q(*,0:tt(2)-30)
   u = u(*,0:tt(2)-30)
   v = v(*,0:tt(2)-30)

   icorig = ic
   ic = ic(*,0:tt(2)-30)
   totqorig = totq
   totuorig = totu
   totvorig = totv

   totq = totq(*,0:tt(2)-30)
   totu = totu(*,0:tt(2)-30)
   totv = totv(*,0:tt(2)-30)

endif else begin

   iorig = i
   tt = size(iorig)
   xdimorig = tt(1)
   ydimorig = tt(2)

endelse

if n_elements(rresize) ne 0 then begin

   i(0:10,*) = median(i)
   q(0:10,*) = 0.
   u(0:10,*) = 0.
   v(0:10,*) = 0.
   
   i(dx_final-10:dx_final,*) = median(i)
   q(dx_final-10:dx_final,*) = 0.
   u(dx_final-10:dx_final,*) = 0.
   v(dx_final-10:dx_final,*) = 0.
   

endif



tt = size(i)
xdim = tt(1)
ydim = tt(2)

a = fft(i,-1)
b = fft(q,-1)
c = fft(u,-1)
d = fft(v,-1)

erg = get_screen_size()

ddimx = erg(0)
ddimy = erg(1)

if xdim gt ddimx then xxdim = ddimx else xxdim = xdim
if 3*ydim gt ddimy then yydim = ddimy else yydim = ydim*3


; define the region in the Fourier power to be removed by up to three
; rectangles 

; Stokes I

ppos_global = fltarr(4,12)
do_pos = fltarr(4)

nregion = fltarr(4)

if n_elements(pposi) ne 0 then begin
   ppos_global(0,0:length(reform(pposi))-1) = pposi
   nregion(0) = length(reform(pposi))
endif else do_pos(0) = 1.

if n_elements(pposq) ne 0 then begin
   ppos_global(1,0:length(reform(pposq))-1) = pposq
   nregion(1) = length(reform(pposq))

endif else do_pos(1) = 1.


if n_elements(pposu) ne 0 then begin
   ppos_global(2,0:length(reform(pposu))-1) = pposu
   nregion(2) = length(reform(pposu))

endif else do_pos(2) = 1.

if n_elements(pposv) ne 0 then begin
   ppos_global(3,0:length(reform(pposv))-1) = pposv
   nregion(3) = length(reform(pposv))

endif else do_pos(3) = 1.

fft_ddata = complexarr(4,xdim,ydim)
fft_ddata(0,*,*) = a
fft_ddata(1,*,*) = b
fft_ddata(2,*,*) = c
fft_ddata(3,*,*) = d

ddata =fltarr(4,xdim,ydim)
ddata(0,*,*) = i
ddata(1,*,*) = q
ddata(2,*,*) = u
ddata(3,*,*) = v

for k = 0,3 do begin

   if do_pos(k) eq 1 and noiquv(k) eq 1 then begin

      window,0,xs = 1*xdim+50,ys = ydim+50,title ='Please mark frequencies to remove: x0, x1, y0, y1'
      tvscl,alog(abs(reform(fft_ddata(k,*,*)))),25,25

;      tvscl,alog(abs(b)),25+xdim,25
;      tvscl,alog(abs(c)),25+2*xdim,25
 ;     plots,[0,0]+xdim+25,[0,ydim+50],/device,thick = 2
 ;     plots,[0,0]+xdim*2+25,[0,ydim+50],/device,thick = 2

   
      if k eq 0 then ttext ='Stokes I'
      if k eq 1 then ttext ='Stokes Q'
      if k eq 2 then ttext ='Stokes U'
      if k eq 3 then ttext ='Stokes V'


      window,1,xs = xxdim,ys = yydim,title ='Filtering '+ttext
      tvscl,congrid(reform(ddata(k,*,*)),xxdim/2.,yydim,/interp),0,0
    ;  tvscl,congrid(u,xxdim/2.,yydim/3.,/interp),0,yydim/3.
    ;  tvscl,congrid(v,xxdim/2.,yydim/3.,/interp),0,2*yydim/3.
      
      cchars = 1.
      ppostotal = 0.
      index = 0
      ppos = fltarr(4)
      finished = 0
      while finished ne 1 do begin
         wset,0
         cursor,x,y,/device,/change
         x = x mod xdim
         if x lt 25 then x = 25
         if y lt 25 then y = 25
         if x ge xdim + 25 then x = xdim +25-1
         if y ge ydim + 25 then y = ydim +25-1
         tvscl,alog(abs(fft_ddata(k,*,*))),25,25
               
         plots,[0,0]+x,[25,25+ydim],/device
         plots,25+[0,3*xdim],[0,0]+y,/device
         xyouts,.1,.1,num2string(x)+','+num2string(y),/normal,charsize = cchars
         
         if !mouse.button eq 1 then begin
            ppos(index) = x-25
            ppos(index+2) = y-25
            index = index + 1
            wait,.7
         endif
         
         if index eq 1 then begin
            draw_rec,[ppos(0)+25,x],[ppos(2)+25,y],/device,color = 240
         endif

         if n_elements(ppostotal) ne 1 then begin
            pppostotal = ppostotal(1:*)
            draw_rec,[pppostotal(0),pppostotal(1)]+25,[pppostotal(2),pppostotal(3)]+25,/device,color = 240
          
           
            draw_rec,25+xdim-[pppostotal(0),pppostotal(1)]+0*xdim,[pppostotal(2),pppostotal(3)]+25,/device,color = 240
            draw_rec,[pppostotal(0),pppostotal(1)]+25,25+ydim-[pppostotal(2),pppostotal(3)],/device,color = 240
           
          
            draw_rec,25+xdim-[pppostotal(0),pppostotal(1)]+0*xdim,25+ydim-[pppostotal(2),pppostotal(3)],/device,color = 240
            if n_elements(ppostotal) ge 8 then begin
               draw_rec,[pppostotal(4),pppostotal(5)]+25,[pppostotal(6),pppostotal(7)]+25,/device,color = 240
              
             
               draw_rec,25+xdim-[pppostotal(4),pppostotal(5)]+0*xdim,[pppostotal(6),pppostotal(7)]+25,/device,color = 240
               draw_rec,[pppostotal(4),pppostotal(5)]+25,25+ydim-[pppostotal(6),pppostotal(7)],/device,color = 240
              
             
               draw_rec,25+xdim-[pppostotal(4),pppostotal(5)]+0*xdim,25+ydim-[pppostotal(6),pppostotal(7)],/device,color = 240
            endif
            if n_elements(ppostotal) ge 12 then begin
               draw_rec,[pppostotal(8),pppostotal(9)]+25,[pppostotal(10),pppostotal(11)]+25,/device,color = 240
           
             
               draw_rec,25+xdim-[pppostotal(8),pppostotal(9)]+0*xdim,[pppostotal(10),pppostotal(11)]+25,/device,color = 240
               draw_rec,[pppostotal(8),pppostotal(9)]+25,25+ydim-[pppostotal(10),pppostotal(11)],/device,color = 240
             
              
               draw_rec,25+xdim-[pppostotal(8),pppostotal(9)]+0*xdim,25+ydim-[pppostotal(10),pppostotal(11)],/device,color = 240
            endif
         endif
  
         if index eq 2 then begin
            draw_rec,[ppos(0),ppos(1)]+25,[ppos(2),ppos(3)]+25,/device,color = 240
           
          
            draw_rec,25+xdim-[ppos(0),ppos(1)]+0*xdim,[ppos(2),ppos(3)]+25,/device,color = 240
            draw_rec,[ppos(0),ppos(1)]+25,25+ydim-[ppos(2),ppos(3)],/device,color = 240
          
           
            draw_rec,25+xdim-[ppos(0),ppos(1)]+0*xdim,25+ydim-[ppos(2),ppos(3)],/device,color = 240
                
            x0 = ppos(0)
            x1 = ppos(1)
            y0 = ppos(2)
            y1 = ppos(3)

            print
            print,'Marked position:'
            print,'x: ',x0,x1
            print,'y: ',y0,y1
            print

            q1 = reform(ddata(k,*,*))
         
            
; apply filter on scan step       
            wset,1    
            if noiquv(k) eq 1 then fft_filter,q1,x0,x1,y0,y1,xdim,ydim
           
            
            if noiquv(k) eq 1 and n_elements(xx0) ne 0 then  begin
               if xx0 ne -1 then fft_filter,q1,xx0,xx1,yy0,yy1,xdim,ydim
            endif

        

            if noiquv(k) eq 1 and n_elements(x2) ne 0 then  fft_filter,q1,x2,x3,y2,y3,xdim,ydim
          

            if noiquv(k) eq 1 and n_elements(x4) ne 0 then  fft_filter,q1,x4,x5,y4,y5,xdim,ydim
          

         
            tvscl,congrid(q1,xxdim/2.,yydim,/interp),xxdim/2.,0
         
      
            xyouts,findgen(2)/2.+.2,.1,['Before','After'],/normal,charsize = cchars*2.
          ;  xyouts,.1,findgen(3)/3.+.1,['Q','U','V'],/normal,charsize = cchars

            print
            ans = read_keyboard('Accept positions (y)es/(n)o ? ')
            if ans eq 'n' then begin
               index = 0
               ppos = fltarr(4)
               xx0 = -1
            endif else begin
               ppostotal = [ppostotal,ppos]
               print
               ans = read_keyboard('Add another region (y)es/(n)o ? ')
               if ans eq 'n' then finished = 1 else begin
                  index = 0
                  if n_elements(ppostotal) ne 1 then begin
                     nn  = n_elements(ppostotal)
                     xx0 = ppostotal(1)
                     xx1 = ppostotal(2)
                     yy0 = ppostotal(3)
                     yy1 = ppostotal(4)
                     
                     if nn ge 8 then begin
                        x2 = ppostotal(5)
                        x3 = ppostotal(6)
                        y2 = ppostotal(7)
                        y3 = ppostotal(8)
                     endif
                     if nn eq 12 then begin
                        x4 = ppostotal(9)
                        x5 = ppostotal(10)
                        y4 = ppostotal(11)
                        y5 = ppostotal(12)
                     endif
                  endif
               endelse 
            endelse
         endif
      endwhile
   endif else begin
      
      x0 = ppos_global(k,0)
      x1 =  ppos_global(k,1)
      y0 =  ppos_global(k,2)
      y1 =  ppos_global(k,3)

      if nregion(k) eq 8 then begin
         x2 = ppos_global(k,4)
         x3 = ppos_global(k,5)
         y2 = ppos_global(k,6)
         y3 = ppos_global(k,7)
      endif

      if nregion(k) eq 12 then begin
         x4 = ppos_global(k,8)
         x5 = ppos_global(k,9)
         y4 = ppos_global(k,10)
         y5 = ppos_global(k,11)
      endif

   endelse

   if n_elements(ppostotal) ge 4 then begin
      ppostotal = ppostotal(1:*)

      nn  = n_elements(ppostotal)
      x0 = ppostotal(0)
      x1 = ppostotal(1)
      y0 = ppostotal(2)
      y1 = ppostotal(3)
      
      if nn ge 8 then begin
         x2 = ppostotal(4)
         x3 = ppostotal(5)
         y2 = ppostotal(6)
         y3 = ppostotal(7)
      endif
      if nn eq 12 then begin
         x4 = ppostotal(8)
         x5 = ppostotal(9)
         y4 = ppostotal(10)
         y5 = ppostotal(11)
      endif
   endif

   
   if n_elements(ppostotal) ge 4 then ppos = ppostotal
   ;print,nn,ppostotal

   if n_elements(ppos) ne 0 then begin
      ppos_global(k,0:length(ppos)-1) = ppos
      nregion(k) = nn
   endif
   

endfor

;if n_elements(pposi) eq 0 then pposi = ppos_global(0,0:nregion(0)-1)
if n_elements(pposi) eq 0 then pposi = ppos_global(0,0:nregion(1)-1)-ppos_global(0,0:nregion(1)-1)
if n_elements(pposq) eq 0 then pposq = ppos_global(1,0:nregion(1)-1)
if n_elements(pposu) eq 0 then pposu = ppos_global(2,0:nregion(2)-1)
if n_elements(pposv) eq 0 then pposv = ppos_global(3,0:nregion(3)-1)


rr = size(ic)

nsteps = rr(1)
window,1,xs = xxdim,ys = yydim
cchars = 1.
window,4,xs = 2*nsteps,ys = 600,title = '2D maps'

outfile = output_dir +strmid(infile,0,strlen(infile)-3)+'fringe_filtered.dat'
outfile1 = outfile + '.sav'

openw,out_unit,outfile,/get_lun

for k = 0,nsteps-1 do begin

   step = k
   read_reduced_spinor,input_dir+infile,step,i,q,u,v,dx_final,dy_final,swap = swap

if n_elements(resize) ne 0 then begin
   iorig = i
   tt = size(iorig)
   i = i(*,0:tt(2)-30)
   q = q(*,0:tt(2)-30)
   u = u(*,0:tt(2)-30)
   v = v(*,0:tt(2)-30)
endif


if n_elements(rresize) ne 0 then begin

   i(0:10,*) = median(i)
   q(0:10,*) = 0.
   u(0:10,*) = 0.
   v(0:10,*) = 0.
   
   i(dx_final-10:dx_final,*) = median(i)
   q(dx_final-10:dx_final,*) = 0.
   u(dx_final-10:dx_final,*) = 0.
   v(dx_final-10:dx_final,*) = 0.
   

endif




   if k eq 0 then begin
      iline = total(i,2)
      if max(iline) eq 0 then iline = iline + 1.
      iline = abs(iline/float(max(iline))-1)

                                ; search for lines automatically 
      n = length(iline)
      ilobes = fltarr(n-4)
      threshh = .2
      ilobeindex = 0.
      for kkk = 5,n-5 do begin
         if iline(kkk) gt max(iline(kkk+1:kkk+4)) and iline(kkk) gt max(iline(kkk-4:kkk-1)) and iline(kkk) gt threshh then begin
            ilobes(ilobeindex) = kkk
            ilobeindex = ilobeindex+1
         endif
      endfor
    
      if n_elements(pos_lines) ne 0 then begin
         ilobes = pos_lines
         ilobeindex = n_elements(ilobes)
      endif
      if n_elements(verbose) ne 0 then print,ilobeindex,' spectral lines found.'
      wdel17 = 0.

; mark center of lines manually
      if ilobeindex ne 2 or n_elements(check_lines) ne 0 then begin
         print,' Automatic detection of spectral lines failed.'
         print,' Please select lines manually:'
         window,17,title = 'Please mark 2 spectral lines'
         wdel17 = 1.
         !p.multi = 0
         plot,iline 
         ilobeindex = 0.
         ilobes = fltarr(n-4)
         while ilobeindex ne 2 do begin
            wset,17
            cursor,x,y,/data,/wait
            if !mouse.button eq 1 then begin
               ilobes(ilobeindex) = x
               ilobeindex = ilobeindex+1
               print,'Line ',ilobeindex
               print,'at ',x
               oplot,[x,x],[0,1]
               wait,.5
            endif
            if ilobeindex eq 2 then begin
               ans = read_keyboard('Selection okay y(es)/n(o) ')
               if ans eq 'n' then begin
                  ilobeindex = 0
                  plot,iline    
               endif
            endif
         endwhile
      endif
      ilobes = ilobes(0:ilobeindex-1)
      ilobes = ilobes(sort(ilobes))

      pos_lines = ilobes
      
   ; location of spectral line to be used for alignment purposes
   ; should be set to Si 1082.7 nm 
      range_telu = [-15,15]+ilobes(0)
     
; location of solar lines for |QUV| maps
      if n_elements(pos_solar) eq 0 then pos_solar = [ilobes(1)-30,ilobes(1)+30.]
      if n_elements(cont_wl) eq 0 then cont_wl = [ilobes(0)+30,ilobes(1)-30]    
      cont_wl = cont_wl(sort(cont_wl))
   endif


   wset,1
   tvscl,congrid(i,xxdim/2.,yydim/4.,/interp),0,0
   tvscl,congrid(q,xxdim/2.,yydim/4.,/interp),0,1*yydim/4.
   tvscl,congrid(u,xxdim/2.,yydim/4.,/interp),0,2*yydim/4.
   tvscl,congrid(v,xxdim/2.,yydim/4.,/interp),0,3*yydim/4.

   ff = where(finite(i) ne 1 or i eq 0)
   if ff(0) ne -1 then i(ff) = 1.
   
   ff = where(finite(q) ne 1)
   if ff(0) ne -1 then q(ff) = 0.
   
   ff = where(finite(u) ne 1)
   if ff(0) ne -1 then u(ff) = 0.

   ff = where(finite(v) ne 1)
   if ff(0) ne -1 then v(ff) = 0.

   x0i = pposi(0)
   x1i = pposi(1)
   y0i = pposi(2)
   y1i = pposi(3)
     
   x0q = pposq(0)
   x1q = pposq(1)
   y0q = pposq(2)
   y1q = pposq(3)
   
   x0u = pposu(0)
   x1u = pposu(1)
   y0u = pposu(2)
   y1u = pposu(3)
   
   x0v = pposv(0)
   x1v = pposv(1)
   y0v = pposv(2)
   y1v = pposv(3)

   
if k eq 0 then begin

;   a1qold = [0.25412798 ,      1.2185188 ,      597.40978 ,     -62.609167]
;   a2qold = [ 0.25413608  ,     1.0549850 ,      611.72383  ,    -82.913758]
;   a3qold = [0.25479463 ,     0.41144606 ,      623.42615 ,     -102.17982]
;   a4qold = [0.25460870 ,    -0.58358276   ,    713.30972 ,     -90.281693]

;   a1uold = [0.25397790      , 1.1263510     ,  355.92936     , -41.982888]
;   a2uold = [     0.25509778  ,    0.89236297 ,      353.43585  ,    -64.099188]
;   a3uold =  [    0.25484014  ,    0.32312233  ,     355.04929  ,    -73.122472]
;   a4uold =   [   0.25473592 ,    -0.65902485   ,    394.23636 ,     -83.261585]
 
;   a1vold = [0.25415280     ,  1.1682747    ,   1808.6765     , -139.32374]
;   a2vold = [   0.25432645 ,     0.99991976 ,      1852.3609   ,   -172.03672]
;   a3vold = [    0.25432836 ,     0.41931054  ,     1917.0835   ,   -281.54993]
;   a4vold = [    0.25492049 ,    -0.62450077   ,    2164.6573  ,    -236.17584]


a1qold = [0.41628822    ,  -1.4362752      , 156.92286    ,   3.4739874]
a2qold = [  0.41083793 ,     -6.0868379     ,  147.39940   ,   -1.3547541]
a3qold = [ 0.42075090 ,     -2.7732895    ,   139.54405   ,    1.3982998]
a4qold = [0.40297051 ,     -3.8492340     ,  197.63742   ,    5.2049914]


a1uold = [ 0.41747206 ,   -0.024263003   ,    628.86309      , 160.67946]
a2uold = [  0.41409934 ,     -5.2914158   ,    674.93277      , 172.34675]
a3uold =  [0.41625883   ,   -5.9588351     ,  686.94508     ,  175.26439]
a4uold =   [ 0.41342113  ,    -5.2113423    ,   655.82428  ,     173.70471]

 
a1vold = [ 0.41674667,      0.20330759     ,  1680.7143  ,     449.52920]
a2vold = [0.41539049  ,    0.61225857     ,  1802.0999    ,   464.81621]
a3vold = [ 0.41560111  ,    0.51319656   ,    1987.6811    ,   505.91549]
a4vold = [  0.41515423  ,    0.53720324 ,      2395.5925    ,   609.72861]



endif

iindex = 0

if n_elements(byline) eq 0 then  begin


   
ii = total(i,1)
ii = ii/max(ii)
   
 if noiquv(0) eq 1 then  fft_filter,i,x0i,x1i,y0i,y1i,xdim,ydim,ii = ii
 if noiquv(1) eq 1 then  fft_filter,q,x0q,x1q,y0q,y1q,xdim,ydim,ii = ii
 if noiquv(2) eq 1 then  fft_filter,u,x0u,x1u,y0u,y1u,xdim,ydim,ii = ii
 if noiquv(3) eq 1 then  fft_filter,v,x0v,x1v,y0v,y1v,xdim,ydim,ii = ii


 if noiquv(0) eq 1 and nregion(0) gt 4 then begin
    x2i = pposi(4)
    x3i = pposi(5)
    y2i = pposi(6)
    y3i = pposi(7)
    fft_filter,i,x2i,x3i,y2i,y3i,xdim,ydim
 endif

 if noiquv(1) eq 1 and nregion(1) gt 4 then begin
    x2q = pposq(4)
    x3q = pposq(5)
    y2q = pposq(6)
    y3q = pposq(7)
    fft_filter,q,x2q,x3q,y2q,y3q,xdim,ydim
 endif

 if noiquv(2) eq 1 and nregion(2) gt 4 then begin
    x2u = pposu(4)
    x3u = pposu(5)
    y2u = pposu(6)
    y3u = pposu(7)
    fft_filter,u,x2u,x3u,y2u,y3u,xdim,ydim
 endif

 if noiquv(3) eq 1 and nregion(3) gt 4 then begin
    x2v = pposv(4)
    x3v = pposv(5)
    y2v = pposv(6)
    y3v = pposv(7)
    fft_filter,v,x2v,x3v,y2v,y3v,xdim,ydim
 endif

 if noiquv(0) eq 1 and nregion(0) eq 12 then begin
    x4i = pposi(8)
    x5i = pposi(9)
    y4i = pposi(10)
    y5i = pposi(11)
    fft_filter,i,x4i,x5i,y4i,y5i,xdim,ydim
 endif

 if noiquv(1) eq 1 and nregion(1) eq 12 then begin
    x4q = pposq(8)
    x5q = pposq(9)
    y4q = pposq(10)
    y5q = pposq(11)
    fft_filter,q,x4q,x5q,y4q,y5q,xdim,ydim
 endif

 if noiquv(2) eq 1 and nregion(2) eq 12 then begin
    x4u = pposu(8)
    x5u = pposu(9)
    y4u = pposu(10)
    y5u = pposu(11)
    fft_filter,u,x4u,x5u,y4u,y5u,xdim,ydim
 endif


 if noiquv(3) eq 1 and nregion(3) eq 12 then begin
    x4v = pposv(8)
    x5v = pposv(9)
    y4v = pposv(10)
    y5v = pposv(11)
    fft_filter,v,x4v,x5v,y4v,y5v,xdim,ydim
 endif

endif else begin

  
rrepeat:


   gg = size(q)
;   poscont = [350,400]

   


   qline1 = smooth(reform(total(q(*,10:gg(2)/4.),2)/(gg(2)/4.-10.+1.)),5,/edge_truncate)
   qline2 = smooth(reform(total(q(*,gg(2)/4.:gg(2)/4.*2.),2)/(gg(2)/4.*2.-gg(2)/4.+1.)),5,/edge_truncate)
   qline3 = smooth(reform(total(q(*,gg(2)/4.*2.:gg(2)/4.*3.),2)/(gg(2)/4.*3.-gg(2)/4.*2.+1.)),5,/edge_truncate)
   qline4 = smooth(reform(total(q(*,gg(2)/4.*3.:gg(2)-10.),2)/(gg(2)-10.-gg(2)/4.*3.+1.)),5,/edge_truncate)

      qooffset = mean(qline1)
      y = qline1(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0) 
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

 ; f = sin(a(0)*x+a(1))*a(2)+a(3)

    ;  y = qline1(50:99)
   ;   xx = findgen(50)+50.
   ;   weights = fltarr(50)+1.
      ;a1 = [.27,.5,max(abs(qline1)),qooffset]


      a1 = [.5,.5,stddev(qline1),qooffset]
    corrq1 = curvefit(xx,y,weights,a1,/noderivative,function_name='sin_var',/double)

      qooffset = mean(qline2)
      y = qline2(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.
     ; y = qline2(50:99)
     ; xx = findgen(50)+50.
     ; weights = fltarr(50)+1.
;      a2 = [.27,.5,max(abs(qline2)),qooffset]

      a2 = [.5,.5,stddev(qline2),qooffset]


      corrq2 = curvefit(xx,y,weights,a2,/noderivative,function_name='sin_var',/double)
 
      qooffset = mean(qline3)
      y =  qline3(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

;qline3(145:175)    
;      xx =  findgen(31)+145. 
;      weights = fltarr(31)+1.
    ;  y = qline3(50:99)
   ;   xx = findgen(50)+50.
  ;    weights = fltarr(50)+1.
;      a3 = [.27,.5,max(abs(qline3)),qooffset]

      a3 = [.5,.5,stddev(qline3),qooffset]


      corrq3 = curvefit(xx,y,weights,a3,/noderivative,function_name='sin_var',/double)
      
      qooffset = mean(qline4)
      y = qline4(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

      ;qline4(145:175)    
      ;xx =  findgen(31)+145. 
      ;weights = fltarr(31)+1.
    ;  y = qline4(50:99)
    ;  xx = findgen(50)+50.
    ;  weights = fltarr(50)+1.
;      a4 = [.27,.5,max(abs(qline4)),qooffset]

      a4 = [.5,.5,stddev(qline4),qooffset]

      corrq4 = curvefit(xx,y,weights,a4,/noderivative,function_name='sin_var',/double)


      if a1(2) lt 0 then begin
         a1(2) = abs(a1(2))
         a1(1) = a1(1)+!pi
      endif
      if a2(2) lt 0 then begin
         a2(2) = abs(a2(2))
         a2(1) = a2(1)+!pi
      endif
      if a3(2) lt 0 then begin
         a3(2) = abs(a3(2))
         a3(1) = a3(1)+!pi
      endif
      if a4(2) lt 0 then begin
         a4(2) = abs(a4(2))
         a4(1) = a4(1)+!pi
      endif

   
      if a1(0) lt 0 then begin
         a1(0) = abs(a1(0))
         a1(2) = abs(a1(2))
         a1(1) = -a1(1) ;mod (2*!pi)
         a1(1) = a1(1)+!pi        
         a1(1) = a1(1) mod (2*!pi)
      endif

      if a2(0) lt 0 then begin
         a2(0) = abs(a2(0))
         a2(2) = abs(a2(2))
         a2(1) = -a2(1) ;mod (2*!pi)
         a2(1) = a2(1)+!pi
         a2(1) = a2(1) mod (2*!pi)
      endif

      if a3(0) lt 0 then begin
         a3(0) = abs(a3(0))
         a3(2) = abs(a3(2))
         a3(1) = -a3(1) ;mod (2*!pi)
         a3(1) = a3(1)+!pi
         a3(1) = a3(1) mod (2*!pi)
      endif

      if a4(0) lt 0 then begin
         a4(0) = abs(a4(0))
         a4(2) = abs(a4(2))
         a4(1) = -a4(1) ;mod (2*!pi)
         a4(1) = a4(1)+!pi
         a4(1) = a4(1) mod (2*!pi)
      endif

;      if a1(0) lt 0.2 or a1(0) gt 0.3 then a1 = a1qold
;      if a2(0) lt 0.2 or a2(0) gt 0.3 then a2 = a2qold
;      if a3(0) lt 0.2 or a3(0) gt 0.3 then a3 = a3qold
;      if a4(0) lt 0.2 or a4(0) gt 0.3 then a4 = a4qold

if iindex eq 0 then begin

      if a1(0) lt 0.4 or a1(0) gt 0.45 then a1 = a1qold
      if a2(0) lt 0.4 or a2(0) gt 0.45 then a2 = a2qold
      if a3(0) lt 0.4 or a3(0) gt 0.45 then a3 = a3qold
      if a4(0) lt 0.4 or a4(0) gt 0.45 then a4 = a4qold

endif

      ; if min( [a1(0),a2(0),a3(0),a4(0)]) lt 0.4 then stop

      a1(1) = a1(1) mod (2*!pi)
      a2(1) = a2(1) mod (2*!pi)
      a3(1) = a3(1) mod (2*!pi)
      a4(1) = a4(1) mod (2*!pi)

      if   a1(1) gt !pi then a1(1) = a1(1)-2*!pi
      if   a2(1) gt !pi then a2(1) = a2(1)-2*!pi
      if   a3(1) gt !pi then a3(1) = a3(1)-2*!pi
      if   a4(1) gt !pi then a4(1) = a4(1)-2*!pi

      if   a1(1) lt (-!pi) then a1(1) = a1(1)+2*!pi
      if   a2(1) lt (-!pi) then a2(1) = a2(1)+2*!pi
      if   a3(1) lt (-!pi) then a3(1) = a3(1)+2*!pi
      if   a4(1) lt (-!pi) then a4(1) = a4(1)+2*!pi

      if a1(1) lt 0 and a2(1) gt 0 then  a2(1) =  a2(1)-2*!pi 
      if a2(1) lt 0 and a3(1) gt 0 then  a3(1) =  a3(1)-2*!pi 
      if a3(1) lt 0 and a4(1) gt 0 then  a4(1) =  a4(1)-2*!pi 

      if iindex eq 0 then begin

      a1qold = a1
      a2qold = a2
      a3qold = a3
      a4qold = a4

      endif
   
      print,'Q'
      print,a1,a2,a3,a4

;      erg0 = poly_fit([75,175,275,375],[a1(0),a2(0),a3(0),a4(0)],2)
;      erg1 =  poly_fit([75,175,275,375],[a1(1),a2(1),a3(1),a4(1)],2)
;      erg2 =  poly_fit([75,175,275,375],[a1(2),a2(2),a3(2),a4(2)],2)
;      erg3 =  poly_fit([75,175,275,375],[a1(3),a2(3),a3(3),a4(3)],2)

 ;     aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
 ;     aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
 ;     aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
 ;     aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)

      poss1 = (gg(2)/4.-10.)/2.
      poss2 = (gg(2)/4.*2.-gg(2)/4.)/2.+gg(2)/4.*1.
      poss3 = (gg(2)/4.*3.-gg(2)/4.*2)/2.+gg(2)/4.*2.
      poss4 = (gg(2)-10.-gg(2)/4.*3.)/2.+gg(2)/4.*3

      erg0 = poly_fit([poss1,poss2,poss3,poss4],[a1(0),a2(0),a3(0),a4(0)],2)
      erg1 =  poly_fit([poss1,poss2,poss3,poss4],[a1(1),a2(1),a3(1),a4(1)],2)
      erg2 =  poly_fit([poss1,poss2,poss3,poss4],[a1(2),a2(2),a3(2),a4(2)],2)
      erg3 =  poly_fit([poss1,poss2,poss3,poss4],[a1(3),a2(3),a3(3),a4(3)],2)

      aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
      aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
      aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
      aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)

 ;     mmax = fltarr(tt(2))
 ;  for kkk = 0,tt(2)-1 do begin & mmax(kkk) = max(q(30:100,kkk)) & endfor
;      mmax = mmax / mean(mmax)

      ccorrq = q - q

   for kkk = 0,tt(2)-1 do begin & sin_var,findgen(tt(1)),[aa0(kkk),aa1(kkk),aa2(kkk),aa3(kkk)],f & ccorrq(*,kkk) = f & endfor

      uline1 = smooth(reform(total(u(*,10:gg(2)/4.),2)/(gg(2)/4.-10.+1.)),5,/edge_truncate)
      uline2 = smooth(reform(total(u(*,gg(2)/4.:gg(2)/4.*2.),2)/(gg(2)/4.*2.-gg(2)/4.+1.)),5,/edge_truncate)
      uline3 = smooth(reform(total(u(*,gg(2)/4.*2.:gg(2)/4.*3.),2)/(gg(2)/4.*3.-gg(2)/4.*2.+1.)),5,/edge_truncate)
      uline4 = smooth(reform(total(u(*,gg(2)/4.*3.:gg(2)-10.),2)/(gg(2)-10.-gg(2)/4.*3.+1.)),5,/edge_truncate)


      
;      uline1 = smooth(reform(total(u(*,50:100),2)/51.),5,/edge_truncate)
;      uline2 = smooth(reform(total(u(*,150:200),2)/51.),5,/edge_truncate)
;      uline3 = smooth(reform(total(u(*,250:300),2)/51.),5,/edge_truncate)
;      uline4 = smooth(reform(total(u(*,350:400),2)/51.),5,/edge_truncate)

      uooffset = mean(uline1) 
     
      y = uline1(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

     ; y = uline1(50:99)
     ; xx = findgen(50)+50.
     ; weights = fltarr(50)+1.
      ;a1 = [.27,.5,max(abs(uline1)),uooffset]
      a1 = [.5,.5,stddev(uline1),uooffset]


      corru1 = curvefit(xx,y,weights,a1,/noderivative,function_name='sin_var',/double)

      uooffset = mean(uline2)

      y = uline2(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

     ; y = uline2(145:175)    
     ; xx =  findgen(31)+145. 
     ; weights = fltarr(31)+1.
    ;  y = uline2(50:99)
    ;  xx = findgen(50)+50.
    ;  weights = fltarr(50)+1.
  ;    a2 = [.27,.5,max(abs(uline2)),uooffset]
      a2 = [.5,.5,stddev(uline2),uooffset]
      corru2 = curvefit(xx,y,weights,a2,/noderivative,function_name='sin_var',/double)
 
      uooffset = mean(uline3)
    ;  y = uline3(50:99)
    ;  xx = findgen(50)+50.
     ; weights = fltarr(50)+1.
 ;     y = uline3(145:175)    
 ;     xx =  findgen(31)+145. 
 ;     weights = fltarr(31)+1.


      y = uline3(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

;      a3 = [.27,.5,max(abs(uline3)),uooffset]

      a3 = [.5,.5,stddev(uline3),uooffset]

      corru3 = curvefit(xx,y,weights,a3,/noderivative,function_name='sin_var',/double)
      
      uooffset = mean(uline4)


      y = uline4(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

;      y = uline4(145:175)    
;      xx =  findgen(31)+145. 
;      weights = fltarr(31)+1.
     ; y = uline4(50:99)
     ; xx = findgen(50)+50.
     ; weights = fltarr(50)+1.
     ; a4 = [.27,.5,max(abs(uline4)),uooffset]
      a4 = [.5,.5,stddev(uline4),uooffset]


      corru4 = curvefit(xx,y,weights,a4,/noderivative,function_name='sin_var',/double)

      if a1(2) lt 0 then begin
         a1(2) = abs(a1(2))
         a1(1) = a1(1)+!pi
      endif
      if a2(2) lt 0 then begin
         a2(2) = abs(a2(2))
         a2(1) = a2(1)+!pi
      endif
      if a3(2) lt 0 then begin
         a3(2) = abs(a3(2))
         a3(1) = a3(1)+!pi
      endif
      if a4(2) lt 0 then begin
         a4(2) = abs(a4(2))
         a4(1) = a4(1)+!pi
      endif


      if   a1(1) lt 0 then a1(1) = a1(1)+2*!pi
      if   a2(1) lt 0 then a2(1) = a2(1)+2*!pi
      if   a3(1) lt 0 then a3(1) = a3(1)+2*!pi
      if   a4(1) lt 0 then a4(1) = a4(1)+2*!pi
    

      if a1(0) lt 0 then begin
         a1(0) = abs(a1(0))
         a1(2) = abs(a1(2))
         a1(1) = -a1(1) ;mod (2*!pi)
         a1(1) = a1(1)+!pi
         a1(1) = a1(1) mod (2*!pi)
      endif

      if a2(0) lt 0 then begin
         a2(0) = abs(a2(0))
         a2(2) = abs(a2(2))
         a2(1) = -a2(1) ;mod (2*!pi)
         a2(1) = a2(1)+!pi
         a2(1) = a2(1) mod (2*!pi)
      endif

      if a3(0) lt 0 then begin
         a3(0) = abs(a3(0))
         a3(2) = abs(a3(2))
         a3(1) = -a3(1) ;mod (2*!pi)
         a3(1) = a3(1)+!pi
         a3(1) = a3(1) mod (2*!pi)
      endif

      if a4(0) lt 0 then begin
         a4(0) = abs(a4(0))
         a4(2) = abs(a4(2))
         a4(1) = -a4(1) ;mod (2*!pi) 
         a4(1) = a4(1)+!pi
         a4(1) = a4(1) mod (2*!pi)
      endif

;      if a1(0) lt 0.2 or a1(0) gt 0.3 then a1 = a1uold
;      if a2(0) lt 0.2 or a2(0) gt 0.3 then a2 = a2uold
;      if a3(0) lt 0.2 or a3(0) gt 0.3 then a3 = a3uold
;      if a4(0) lt 0.2 or a4(0) gt 0.3 then a4 = a4uold
  if iindex eq 0 then begin

      if a1(0) lt 0.4 or a1(0) gt 0.45 then a1 = a1uold
      if a2(0) lt 0.4 or a2(0) gt 0.45 then a2 = a2uold
      if a3(0) lt 0.4 or a3(0) gt 0.45 then a3 = a3uold
      if a4(0) lt 0.4 or a4(0) gt 0.45 then a4 = a4uold
  endif

      a1(1) = a1(1) mod (2*!pi)
      a2(1) = a2(1) mod (2*!pi)
      a3(1) = a3(1) mod (2*!pi)
      a4(1) = a4(1) mod (2*!pi)

      if   a1(1) gt !pi then a1(1) = a1(1)-2*!pi
      if   a2(1) gt !pi then a2(1) = a2(1)-2*!pi
      if   a3(1) gt !pi then a3(1) = a3(1)-2*!pi
      if   a4(1) gt !pi then a4(1) = a4(1)-2*!pi

      if   a1(1) lt (-!pi) then a1(1) = a1(1)+2*!pi
      if   a2(1) lt (-!pi) then a2(1) = a2(1)+2*!pi
      if   a3(1) lt (-!pi) then a3(1) = a3(1)+2*!pi
      if   a4(1) lt (-!pi) then a4(1) = a4(1)+2*!pi

      if a1(1) lt 0 and a2(1) gt 0 then  a2(1) =  a2(1)-2*!pi 
      if a2(1) lt 0 and a3(1) gt 0 then  a3(1) =  a3(1)-2*!pi 
      if a3(1) lt 0 and a4(1) gt 0 then  a4(1) =  a4(1)-2*!pi 

      if iindex eq 0 then begin

      a1uold = a1
      a2uold = a2
      a3uold = a3
      a4uold = a4

endif
      print,'U'
      print,a1,a2,a3,a4

   ;   erg0 = poly_fit([75,175,275,375],[a1(0),a2(0),a3(0),a4(0)],2)
   ;   erg1 =  poly_fit([75,175,275,375],[a1(1),a2(1),a3(1),a4(1)],2)
   ;   erg2 =  poly_fit([75,175,275,375],[a1(2),a2(2),a3(2),a4(2)],2)
   ;   erg3 =  poly_fit([75,175,275,375],[a1(3),a2(3),a3(3),a4(3)],2)

   ;   aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
   ;   aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
   ;   aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
   ;   aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)
  
      erg0 = poly_fit([poss1,poss2,poss3,poss4],[a1(0),a2(0),a3(0),a4(0)],2)
      erg1 =  poly_fit([poss1,poss2,poss3,poss4],[a1(1),a2(1),a3(1),a4(1)],2)
      erg2 =  poly_fit([poss1,poss2,poss3,poss4],[a1(2),a2(2),a3(2),a4(2)],2)
      erg3 =  poly_fit([poss1,poss2,poss3,poss4],[a1(3),a2(3),a3(3),a4(3)],2)

      aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
      aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
      aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
      aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)



      ccorru = u - u

   for kkk = 0,tt(2)-1 do begin & sin_var,findgen(tt(1)),[aa0(kkk),aa1(kkk),aa2(kkk),aa3(kkk)],f & ccorru(*,kkk) = f & endfor


      vline1 = smooth(reform(total(v(*,10:gg(2)/4.),2)/(gg(2)/4.-10.+1.)),5,/edge_truncate)
      vline2 = smooth(reform(total(v(*,gg(2)/4.:gg(2)/4.*2.),2)/(gg(2)/4.*2.-gg(2)/4.+1.)),5,/edge_truncate)
      vline3 = smooth(reform(total(v(*,gg(2)/4.*2.:gg(2)/4.*3.),2)/(gg(2)/4.*3.-gg(2)/4.*2.+1.)),5,/edge_truncate)
      vline4 = smooth(reform(total(v(*,gg(2)/4.*3.:gg(2)-10.),2)/(gg(2)-10.-gg(2)/4.*3.+1.)),5,/edge_truncate)

      ;      vline1 = smooth(reform(total(v(*,50:100),2)/51.),5,/edge_truncate)
      ;vline2 = smooth(reform(total(v(*,150:200),2)/51.),5,/edge_truncate)
      ;vline3 = smooth(reform(total(v(*,250:300),2)/51.),5,/edge_truncate)
      ;vline4 = smooth(reform(total(v(*,350:400),2)/51.),5,/edge_truncate)

      vooffset = mean(vline1)
      ;y = vline1(145:175)  ; (50:99)
      ;xx = findgen(31)+145.   ; findgen(50)+50.
      ;weights = fltarr(31)+1. ; fltarr(50)+1.
   
      y = vline1(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.

;      a1 = [.27,.5,max(abs(vline1)),vooffset]

      a1 = [.5,.5,stddev(vline1),vooffset]


      corrv1 = curvefit(xx,y,weights,a1,/noderivative,function_name='sin_var',/double)

      vooffset = mean(vline2)
;      y = vline2(145:175)  ;(50:99)
;      xx =  findgen(31)+145.   ;findgen(50)+50.
;      weights = fltarr(31)+1. ;fltarr(50)+1.
 
      y = vline2(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.
;      a2 = [.27,.5,max(abs(vline2)),vooffset]

      a2 = [.5,.5,stddev(vline2),vooffset]


      corrv2 = curvefit(xx,y,weights,a2,/noderivative,function_name='sin_var',/double)
 
      vooffset = mean(vline3)
;      y = vline3(145:175)  ;(50:99)
;      xx =  findgen(31)+145.   ;findgen(50)+50.
;      weights = fltarr(31)+1. ;fltarr(50)+1.

      y = vline3(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.


     ; a3 = [.27,.5,max(abs(vline3)),vooffset]
      a3 = [.5,.5,stddev(vline3),vooffset]


      corrv3 = curvefit(xx,y,weights,a3,/noderivative,function_name='sin_var',/double)
      
      vooffset = mean(vline4)
;      y = vline4(145:175)  ;(50:99)
;      xx =  findgen(31)+145.   ;findgen(50)+50.
;      weights = fltarr(31)+1. ;fltarr(50)+1.

      y = vline4(poscont(0):poscont(1))    
      xx =  findgen(poscont(1)-poscont(0)+1)+poscont(0)  
      weights = fltarr(poscont(1)-poscont(0)+1)+1.


;      a4 = [.27,.5,max(abs(vline4)),vooffset]

      a4 = [.5,.5,stddev(vline4),vooffset]

      corrv4 = curvefit(xx,y,weights,a4,/noderivative,function_name='sin_var',/double)



      if a1(2) lt 0 then begin
         a1(2) = abs(a1(2))
         a1(1) = a1(1)+!pi
      endif
      if a2(2) lt 0 then begin
         a2(2) = abs(a2(2))
         a2(1) = a2(1)+!pi
      endif
      if a3(2) lt 0 then begin
         a3(2) = abs(a3(2))
         a3(1) = a3(1)+!pi
      endif
      if a4(2) lt 0 then begin
         a4(2) = abs(a4(2))
         a4(1) = a4(1)+!pi
      endif

      if   a1(1) gt !pi then a1(1) = a1(1)-2*!pi
      if   a2(1) gt !pi then a2(1) = a2(1)-2*!pi
      if   a3(1) gt !pi then a3(1) = a3(1)-2*!pi
      if   a4(1) gt !pi then a4(1) = a4(1)-2*!pi

      if a1(0) lt 0 then begin
         a1(0) = abs(a1(0))
         a1(2) = abs(a1(2))
         a1(1) = -a1(1) ;mod (2*!pi)
         a1(1) = a1(1)+!pi
         a1(1) = a1(1) mod (2*!pi)
      endif

      if a2(0) lt 0 then begin
         a2(0) = abs(a2(0))
         a2(2) = abs(a2(2))
         a2(1) = -a2(1) ;mod (2*!pi)
         a2(1) = a2(1)+!pi
         a2(1) = a2(1) mod (2*!pi)
      endif

      if a3(0) lt 0 then begin
         a3(0) = abs(a3(0))
         a3(2) = abs(a3(2))
         a3(1) = -a3(1) ;mod (2*!pi)
         a3(1) = a3(1)+!pi
         a3(1) = a3(1) mod (2*!pi)
      endif

      if a4(0) lt 0 then begin
         a4(0) = abs(a4(0))
         a4(2) = abs(a4(2))
         a4(1) = -a4(1)
         a4(1) = a4(1)+!pi
         a4(1) = a4(1) mod (2*!pi)
      endif


;      if a1(0) lt 0.2  or a1(0) gt 0.3 then a1 = a1vold
;      if a2(0) lt 0.2 or a2(0) gt 0.3 then a2 = a2vold
;      if a3(0) lt 0.2  or a3(0) gt 0.3 then a3 = a3vold
;      if a4(0) lt 0.2 or a4(0) gt 0.3 then a4 = a4vold

      if iindex eq 0 then begin

      if a1(0) lt 0.4  or a1(0) gt 0.45 then a1 = a1vold
      if a2(0) lt 0.4 or a2(0) gt 0.45 then a2 = a2vold
      if a3(0) lt 0.4  or a3(0) gt 0.45 then a3 = a3vold
      if a4(0) lt 0.4 or a4(0) gt 0.45 then a4 = a4vold
   endif


      a1(1) = a1(1) mod (2*!pi)
      a2(1) = a2(1) mod (2*!pi)
      a3(1) = a3(1) mod (2*!pi)
      a4(1) = a4(1) mod (2*!pi)


      if   a1(1) gt !pi then a1(1) = a1(1)-2*!pi
      if   a2(1) gt !pi then a2(1) = a2(1)-2*!pi
      if   a3(1) gt !pi then a3(1) = a3(1)-2*!pi
      if   a4(1) gt !pi then a4(1) = a4(1)-2*!pi

      if   a1(1) lt (-!pi) then a1(1) = a1(1)+2*!pi
      if   a2(1) lt (-!pi) then a2(1) = a2(1)+2*!pi
      if   a3(1) lt (-!pi) then a3(1) = a3(1)+2*!pi
      if   a4(1) lt (-!pi) then a4(1) = a4(1)+2*!pi

      if a1(1) lt 0 and a2(1) gt 0 then  a2(1) =  a2(1)-2*!pi 
      if a2(1) lt 0 and a3(1) gt 0 then  a3(1) =  a3(1)-2*!pi 
      if a3(1) lt 0 and a4(1) gt 0 then  a4(1) =  a4(1)-2*!pi 

if iindex eq 0 then begin

      a1vold = a1
      a2vold = a2
      a3vold = a3
      a4vold = a4

endif


      print,'V'
      print,a1,a2,a3,a4

;      erg0 = poly_fit([75,175,275,375],[a1(0),a2(0),a3(0),a4(0)],2)
;      erg1 =  poly_fit([75,175,275,375],[a1(1),a2(1),a3(1),a4(1)],2)
;      erg2 =  poly_fit([75,175,275,375],[a1(2),a2(2),a3(2),a4(2)],2)
;      erg3 =  poly_fit([75,175,275,375],[a1(3),a2(3),a3(3),a4(3)],2)

;      aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
;      aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
;      aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
;      aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)


      erg0 = poly_fit([poss1,poss2,poss3,poss4],[a1(0),a2(0),a3(0),a4(0)],2)
      erg1 =  poly_fit([poss1,poss2,poss3,poss4],[a1(1),a2(1),a3(1),a4(1)],2)
      erg2 =  poly_fit([poss1,poss2,poss3,poss4],[a1(2),a2(2),a3(2),a4(2)],2)
      erg3 =  poly_fit([poss1,poss2,poss3,poss4],[a1(3),a2(3),a3(3),a4(3)],2)

      aa0 = erg0(0)+findgen(tt(2))*erg0(1)+findgen(tt(2))^2.*erg0(2)
      aa1 = erg1(0)+findgen(tt(2))*erg1(1)+findgen(tt(2))^2.*erg1(2)
      aa2 = erg2(0)+findgen(tt(2))*erg2(1)+findgen(tt(2))^2.*erg2(2)
      aa3 = erg3(0)+findgen(tt(2))*erg3(1)+findgen(tt(2))^2.*erg3(2)

      ccorrv = v - v

   for kkk = 0,tt(2)-1 do begin & sin_var,findgen(tt(1)),[aa0(kkk),aa1(kkk),aa2(kkk),aa3(kkk)],f & ccorrv(*,kkk) = f & endfor


      
      if iindex eq 0 then begin
;         print,iindex
;         stop
         q = q -ccorrq
         v = v -ccorrv
      endif

      u = u -ccorru

      
      iindex = iindex + 1


;paused

;      if iindex eq 1 then goto,rrepeat

   endelse





 ff = where(finite(i) ne 1 or i eq 0)
 if ff(0) ne -1 then i(ff) = 1.
 
 ff = where(finite(q) ne 1)
 if ff(0) ne -1 then q(ff) = 0.
 
 ff = where(finite(u) ne 1)
 if ff(0) ne -1 then u(ff) = 0.

 ff = where(finite(v) ne 1)
 if ff(0) ne -1 then v(ff) = 0.

   tvscl,congrid(i,xxdim/2.,yydim/4.,/interp),xxdim/2.,0
   tvscl,congrid(q/i,xxdim/2.,yydim/4.,/interp),xxdim/2.,1*yydim/4
   tvscl,congrid(u/i,xxdim/2.,yydim/4.,/interp),xxdim/2.,2*yydim/4.
   tvscl,congrid(v/i,xxdim/2.,yydim/4.,/interp),xxdim/2.,3*yydim/4.

   xyouts,.1,findgen(4)/4.+.1,['I','Q','U','V'],/normal,charsize = cchars
   xyouts,findgen(2)/2.+.2,.1,['Before','After'],/normal,charsize = cchars*2.
   
   ic(k,*) = total(i(cont_wl(0):cont_wl(1),*),1)/float(cont_wl(1)-cont_wl(0)+1.)

   qq = median(q/i,5)
   uu = median(u/i,5)
   vv = median(v/i,5)

 ;  totq(k,*) = total( abs ( q(pos_solar(0):pos_solar(1),*)/i(pos_solar(0):pos_solar(1),*)),1)
 ;  totu(k,*) = total( abs ( u(pos_solar(0):pos_solar(1),*)/i(pos_solar(0):pos_solar(1),*)),1)
;   totv(k,*) = total( abs (
;   v(pos_solar(0):pos_solar(1),*)/i(pos_solar(0):pos_solar(1),*)),1)

   totq(k,*) = total( abs ( qq(pos_solar(0):pos_solar(1),*)),1)
   totu(k,*) = total( abs ( uu(pos_solar(0):pos_solar(1),*)),1)
   totv(k,*) = total( abs ( vv(pos_solar(0):pos_solar(1),*)),1)

   if n_elements(resize) ne 0 then begin
      itemp = iorig - iorig
      qtemp = iorig - iorig
      utemp = iorig - iorig
      vtemp = iorig - iorig

      itemp(*,0:tt(2)-30) = i
      qtemp(*,0:tt(2)-30) = q
      utemp(*,0:tt(2)-30) = u
      vtemp(*,0:tt(2)-30) = v

      i = itemp
      q = qtemp
      u = utemp
      v = vtemp

   endif

   a = fltarr(xdimorig,ydimorig,4)
   a(*,*,0) = i
   a(*,*,1) = q
   a(*,*,2) = u
   a(*,*,3) = v

   writeu,out_unit,a


   if n_elements(plt) ne 0 then begin
      wset,4
      tvscl,congrid(ic,nsteps,300,/interp) > 10 ,0,0
      tvscl,alog(congrid(totq,nsteps,300,/interp) > 0.0001),nsteps,0
      tvscl,alog(congrid(totu,nsteps,300,/interp) > 0.0001),0,300
      tvscl,alog(congrid(totv,nsteps,300,/interp) > 0.0001),nsteps,300
   endif


   print,nsteps-1-k

endfor

free_lun,out_unit

if n_elements(resize) ne 0 then begin

   icorig(*,0:tt(2)-30) = ic
   totqorig(*,0:tt(2)-30) = totq
   totuorig(*,0:tt(2)-30) = totu
   totvorig(*,0:tt(2)-30) = totv

   ic = icorig
   totq = totqorig
   totu = totuorig
   totv = totvorig

endif


; save overview maps and other info
save,ic,totq,totu,totv,sshifts,v2q,v2u,llinepos,pos_solar,pos_lines,dx_final,dy_final,n_slits,int_only,x0,x1,y0,y1,filename = outfile1



end
