function vtt_ten,th1,th2,th3,winret,winang,exret,exang,offout,coel1rs,coel1rp,coel1ret,coel2rs,coel2rp,coel2ret,prirs,prirp,priret,calrs,calrp,calret,xi,th_pol_C1,th_C1_pol,th_pol_C2,alpha,beta,ideal=ideal,polarizer = polarizer

;+
;==============================================================================
;
;	function :  vtt_ten
;
;	purpose  :  return the unnormalized telescope matrix for the
;		    vacuum tower telescope Teneriffa
;
;       written  :  cbeck@kis, 12/01
;
;==============================================================================
;
;       Check number of parameters.
;
;==============================================================================
if n_params() lt 24 then begin
	print
	print, "usage:	T = vtt_ten( th1, th2, th3          $"
	print, "                     , winret, winang       $"
	print, "                     , exret, exang         $"
	print, "                     , offout               $"
	print, "                     , coel1rs,coel1rp,coel1ret$"
	print, "                     , coel2rs,coel2rp,coel2ret$"
	print, "                     , prirs, prirp, priret,xi$"
        print, "                     , th_pol_C1,th_C1_pol,th_pol_C2,"
        print, "                     , /polarizer, /ideal )"
	print
	print, "        Return the 4x4 vacuum tower telescope matrix."
        print	
	print, "	Arguments"
        print, "                th1     : rotation angle between RF0 and first"
        print, "                          coelostat mirror"
        print, "                th2     : rotation angle between C1 and C2"
        print, "                th3     : rotation angle between C2 and main"
        print, "                winret  : retardance of entry window"
        print, "                winang  : position angle of entry window"
        print, "                exret   : retardance of exit window"
        print, "                exang   : position angle of exit window"
        print, "                offout  : rotation between deflection mirror"
        print, "                          and polarimeter fiducial axis"
        print, "                coel1rs : reflectivity perpendicular to plain"
        print, "                          of incidence, C1"
        print, "                coel1rp : reflectivity in the plain of"
        print, "                          incidence, C1"
        print, "                coel1ret: retardance of first mirror C1"
        print, "                coel2rs : reflectivity perpendicular to plain"
        print, "                          of incidence, C2"
        print, "                coel2rp : reflectivity in the plain of"
        print, "                          incidence, C2"
        print, "                coel2ret: retardance of second mirror C2"
        print, "                prirs   : reflectivity perpendicular to plain"
        print, "                          of incidence, main and deflection"
        print, "                prirp   : reflectivity in the plain of"
        print, "                          incidence, main and deflection"
        print, "                priret  : retardance of main and deflection"
        print, "                          mirror"
        print, "                xi     : position angle of polarizer"
        print, "                on window: W to N, on C1: N to E"
        print, "                th_pol_C1 : rotation between polarizer and C1"
        print, "                th_C1_pol : rotation between reflected C1 and"
        print, "                            linear polarizer"
        print, "                th_pol_C2 : rotation between polarizer and C2"
	print
        print, "       Keywords"
        print, "                /ideal : use ideal mirrors diag(1,1,-1,-1)"
        print, "                polarizer = 1 : put polarizer on C1"
        print, "                polarizer = 2 : put polarizer on window"
        print
        print, "       Important : all angles have to be given in degrees !"
        print
	return,0
    endif
;=============================================================================
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This function returns the unnormalized telescope matrix for the
;vacuum tower telescope. 
;
;Output:
;
;       vtt_ten	Four by four telescope matrix.
;
;Input:		(All angles are in degrees).
;
;	th1       Rotation angle between sun and first coelostat mirror.
;	th2       Rotation angle between first and second coelostat mirror.
;	th3       Rotation angle between 2. coelostat and main mirror.
;	winret	  Retardance of entrance window.
;	winang	  Angle of entrance retardance.
;	exret	  Exit port retardance.
;	exang	  Exit port angle.
;	offout	  Polarimeter rotation on exit port.
;	coel1rs	  Mirror 1 reflectance perpendicular to plain of incidence.
;	coel1rp	  Mirror 1 reflectance in the plain of incidence.
;	coel1ret  Phase difference between coel1rs and coel1rp.
;	coel2rs	  Mirror 2 reflectance perpendicular to plain of incidence.
;	coel2rp	  Mirror 2 reflectance in the plain of incidence.
;	coel2ret  Phase difference between coel2rs and coel2rp.
;	prirs	  Primary mirror reflectance perpendicular to
;		  plain of incidence.
;	prirp	  Primary mirror reflectance in the
;		  plain of incidence.
;	priret    Phase difference between prirs and prirp
;       xi        angle of polarizer to rotation axis of C1
;       chi       angle of polarizer to W on entrance window
;       th_pol_C1 angle between polarizer fiducial axis and C1
;       th_C1_pol angle between reflected C1-axis and polarizer
;       th_pol_C2 angle between polarizer fiducial axis and C2
;
;       written : cbeck@kis 12/01
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------
			;
			;Convert degrees to radians.
			;
degtorad = !pi/180.

if n_elements(polarizer) ne 0 then polpos = polarizer
if n_elements(polarizer) eq 0 then polpos = 0

;if polpos eq 1 then th3 = th3-90
;if polpos eq 2 then th3 = th3+90

th_1     = th1 * degtorad
th_2     = th2 * degtorad
th_3     = (th3+90) * degtorad ;+90

th_4     = (th_pol_C1) * degtorad 
th_5     = (th_C1_pol) * degtorad 
th_6     = (th_pol_C2) * degtorad 

rwinret  = winret * degtorad
rwinang  = winang * degtorad
rexret   = exret  * degtorad
rexang   = exang  * degtorad
roffout  = offout * degtorad
rret1    = coel1ret * degtorad
rret2    = coel2ret * degtorad
rpriret  = priret   * degtorad
calret   = calret   * degtorad

;------------------------------
; use ideal mirrors 
;------------------------------
if n_elements(ideal) ne 0 then begin
coel1rp = 1
coel1rs = 1
coel2rp = 1
coel2rs = 1
prirs = 1
prirp = 1
rret1    = !pi
rret2    = !pi
rpriret = !pi
endif
;------------------------------

;-----------------------------------------------------
; calculate all needed matrices
;-----------------------------------------------------

;------------------------------
			;
			;Entrance window matrix (retarder).
			;
sd = sin( rwinret )
cd = cos( rwinret )
s  = sin( 2.*rwinang )
c  = cos( 2.*rwinang )
s2 = s*s
c2 = c*c
winmtx = setmtxright(                  $
  1.,          0.,          0.,    0.  $
, 0.,    c2+s2*cd, c*s*(1.-cd), -s*sd  $
, 0., c*s*(1.-cd),    s2+c2*cd,  c*sd  $
, 0.,        s*sd,       -c*sd,    cd  )
;------------------------------
			;
			;Exit window matrix (retarder).
			;
sd = sin( rexret )
cd = cos( rexret )
s  = sin( 2.*rexang )
c  = cos( 2.*rexang )
s2 = s*s
c2 = c*c
exmtx = setmtxright(                   $
  1.,          0.,          0.,    0.  $
, 0.,    c2+s2*cd, c*s*(1.-cd), -s*sd  $
, 0., c*s*(1.-cd),    s2+c2*cd,  c*sd  $
, 0.,        s*sd,       -c*sd,    cd  )

;------------------------------
			;
			; first coelostat mirror matrix.
			;
rt = sqrt( coel1rs*coel1rp )
sd = sin( rret1 )
cd = cos( rret1 )
mir1mtx = setmtxright(                             $
  .5*(coel1rp+coel1rs), .5*(coel1rp-coel1rs),       0.,       0.  $
, .5*(coel1rp-coel1rs), .5*(coel1rp+coel1rs),       0.,       0.  $
,         0.,         0.,    rt*cd,    rt*sd  $
,         0.,         0.,   -rt*sd,    rt*cd  )

;------------------------------
			;
			; second coelostat mirror matrix.
			;
rt = sqrt( coel2rs*coel2rp )
sd = sin( rret2 )
cd = cos( rret2 )
mir2mtx = setmtxright(                             $
  .5*(coel2rp+coel2rs), .5*(coel2rp-coel2rs),       0.,       0.  $
, .5*(coel2rp-coel2rs), .5*(coel2rp+coel2rs),       0.,       0.  $
,         0.,         0.,    rt*cd,    rt*sd  $
,         0.,         0.,   -rt*sd,    rt*cd  )
;------------------------------
			;
			;Primary and additional mirror matrix.
			;
rt = sqrt( prirs*prirp )
sd = sin( rpriret )
cd = cos( rpriret )
primtx = setmtxright(                                         $
  .5*(prirp+prirs), .5*(prirp-prirs),       0.,      0.  $
, .5*(prirp-prirs), .5*(prirp+prirs),       0.,      0.  $
,               0.,               0.,    rt*cd,   rt*sd  $
,               0.,               0.,   -rt*sd,   rt*cd  )
;------------------------------
			;
			;Rotation matrix between 1. and 2. coelostat mirrors.
			;
s = sin( 2.*th_2 )
c = cos( 2.*th_2 )
elazmtx = setmtxright(   $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )
;------------------------------
			;
			;Rotation matrix between 2. and primary mirrors.
			;
s = sin( 2.*th_3 )
c = cos( 2.*th_3 )
azprimtx = setmtxright(   $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )	
;------------------------------
			;
			;Rotation matrix between sun and 1. mirror.
			;
s = sin( 2.*th_1 )
c = cos( 2.*th_1 )
sunfirstmtx = setmtxright(   $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )	
;------------------------------
			;
			;Rotation from additional mirror to polarimeter.
			;
s = sin( 2.*roffout )
c = cos( 2.*roffout )
offmtx = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )
;------------------------------

;------------------------------
; put polarizer on first mirror C1
;------------------------------
if polpos eq 1 then begin
xi_rad   = xi * degtorad
			;
			; Rotation from polarizer to first mirror.
			;
s = sin( 2.*th_4 )
c = cos( 2.*th_4 )
pol_C1 = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )
;------------------------------
			;
			; Rotation from first mirror (reflected) to polarizer.
			;
s = sin( 2.*th_5 )
c = cos( 2.*th_5 )
C1_pol = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )
;------------------------------
			;
			; Rotation from polarizer to second mirror.
			;
s = sin( 2.*th_6 )
c = cos( 2.*th_6 )
pol_C2 = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )
;------------------------------
			;
			; polarizer input depending on angle xi, incoming
			;
s = sin(2*xi_rad)
c = cos(2*xi_rad)

pol_rot = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )

alphaold = alpha

beta = 0.
alpha = 0.

pol_mat_temp = setmtxright(1,cos(2*alpha),0,0,cos(2*alpha),1,0,0,0,0,sin(2*alpha)*cos(beta),-sin(2*alpha)*sin(beta),0,0,sin(2*alpha)*sin(beta),sin(2*alpha)*cos(beta))*.5

pol_rot1 = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  -s, 0.  $
, 0., s,  c, 0.  $
, 0., 0., 0., 1.  )

pol_mat_in = pol_rot1 ## pol_mat_temp ## pol_rot 

;------------------------------
			;
			; polarizer matrix for beam reflected on C1
			; xi -> -xi
s = sin(2*xi_rad)
c = cos(2*xi_rad)

pol_rot = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  -s, 0.  $
, 0., s,  c, 0.  $
, 0., 0., 0., 1.  )

pol_mat_temp = setmtxright(1,cos(2*alpha),0,0,cos(2*alpha),1,0,0,0,0,sin(2*alpha)*cos(beta),-sin(2*alpha)*sin(beta),0,0,sin(2*alpha)*sin(beta),sin(2*alpha)*cos(beta))*.5

pol_rot1 = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )

pol_mat_ref = pol_rot1 ## pol_mat_temp ## pol_rot 

;------------------------------
; cal unit mounting
rt = sqrt( calrs*calrp )
sd = sin( calret )
cd = cos( calret )
calmtx = setmtxright(                             $
  .5*(calrp+calrs), .5*(calrp-calrs),       0.,       0.  $
, .5*(calrp-calrs), .5*(calrp+calrs),       0.,       0.  $
,         0.,         0.,    rt*cd,    rt*sd  $
,         0.,         0.,   -rt*sd,    rt*cd  )

endif

if polpos eq 2 then begin
xi_rad   = xi * degtorad

s = sin(2*xi_rad)
c = cos(2*xi_rad)

pol_rot = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  s, 0.  $
, 0., -s,  c, 0.  $
, 0., 0., 0., 1.  )

pol_mat_temp = setmtxright(1,cos(2*alpha),0,0,cos(2*alpha),1,0,0,0,0,sin(2*alpha),0,0,0,0,sin(2*alpha))*.5

pol_rot1 = setmtxright(  $
  1., 0., 0., 0.  $
, 0.,  c,  -s, 0.  $
, 0., s,  c, 0.  $
, 0., 0., 0., 1.  )

pol_mat1 = pol_rot1##pol_mat_temp##pol_rot

endif

;------------------------------
                        ;
			;Return the instrument matrix (matrix product ##).
			;

;print,pol_mat*2
;print,pol_mat_ref*2

;print,exmtx,' exit window'
;print,''
;print,winmtx,'  entry window'
;print,''
;print,primtx,'  primary and additional mirror'
;print,''
;print,mir1mtx,'  first mirror'
;print,coel1ret,' retardance'
;print,''
;print,mir2mtx,'  second mirror'
;print,''
;print,elazmtx,'  rot coel1 -> coel2'
;print,''
;print,azprimtx,'  rot coel2 -> main'
;print,''
;print,sunfirstmtx,'  rot sun -> coel1'

T =  offmtx ## exmtx ## primtx ## primtx ## winmtx## azprimtx ##mir2mtx ## elazmtx ## mir1mtx ## sunfirstmtx    

;------------------------------
; with polarizer
;------------------------------
if polpos eq 1 then begin

beta = alphaold

;beta = 0

T = offmtx ## exmtx ## primtx ## primtx ## winmtx ## azprimtx  ##mir2mtx ##(( pol_C2 ## pol_mat_ref ## C1_pol ## mir1mtx ## pol_C1 ## pol_mat_in)+calmtx*beta)

endif

if polpos eq 2 then begin
T =  offmtx ## exmtx ## primtx ## primtx## winmtx ## pol_mat1 ## azprimtx ## mir2mtx ## elazmtx ## mir1mtx ## sunfirstmtx

endif

;-----------------------------------------------------------
; result unnormalized telescope matrix
;-----------------------------------------------------------
return,T

end
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
